=== modified file 'NEWS'
--- NEWS	2007-02-12 15:57:20 +0000
+++ NEWS	2007-02-15 15:11:31 +0000
@@ -35,6 +35,23 @@
     * ``bzr help checkouts`` descibes what checkouts are and some possible
       uses of them. (James Westby, Aaron Bentley)
 
+    * A new ``-d`` option to push, pull and merge overrides the default 
+      directory.  (Martin Pool)
+
+    * Branch format 6: smaller, and potentially faster than format 5.  Supports
+      "append_history_only" mode, where the log view and revnos do not change,
+      except by being added to.  Stores policy settings in
+      ".bzr/branch/branch.conf".
+
+    * append_only branches:  Format 6 branches may be configured so that log
+      view and revnos are always consistent.  Either create the branch using
+      "bzr init --append-revisions-only" or edit the config file as descriped
+      in docs/configuration.txt.
+
+    * rebind: Format 6 branches retain the last-used bind location, so if you
+      "bzr unbind", you can "bzr bind" to bind to the previously-selected
+      bind location.
+
   INTERNALS:
 
     * Reserved ids (any revision-id ending in a colon) are rejected by
@@ -98,6 +115,8 @@
     * ``bzr log branch/file`` works for local treeless branches
       (Aaron Bentley, #84247)
 
+    * Fix problem with UNC paths on Windows 98. (Alexander Belchenko, #84728)
+
   TESTING:
 
     * New ``--first`` option to ``bzr selftest`` to run specified tests

=== modified file 'bzr'
--- bzr	2007-01-24 20:40:20 +0000
+++ bzr	2007-02-16 06:06:14 +0000
@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -96,6 +96,16 @@
 
     if profiling:
         profile_imports.log_stack_info(sys.stderr)
-    sys.exit(exit_val)
+
+    # By this point we really have completed everything we want to do, and
+    # there's no point doing any additional cleanup.  Abruptly exiting here
+    # stops any background threads getting into trouble as code is unloaded,
+    # and it may also be slightly faster, through avoiding gc of objects that
+    # are just about to be discarded anyhow.  This does mean that atexit hooks
+    # won't run but we don't use them.  Also file buffers won't be flushed,
+    # but our policy is to always close files from a finally block. -- mbp 20070215
+    sys.stdout.flush()
+    # stderr isn't buffered
+    os._exit(exit_val)
 else:
     pass    # should this give an error? - it can't be used as a lib

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2007-02-13 20:33:57 +0000
+++ bzrlib/branch.py	2007-02-17 01:42:57 +0000
@@ -225,7 +225,9 @@
         try:
             if last_revision is None:
                 pb.update('get source history')
-                last_revision = from_branch.last_revision_info()[1]
+                last_revision = from_branch.last_revision()
+                if last_revision is None:
+                    last_revision = _mod_revision.NULL_REVISION
             return self.repository.fetch(from_branch.repository,
                                          revision_id=last_revision,
                                          pb=nested_pb)
@@ -242,6 +244,11 @@
         """
         return None
     
+    def get_old_bound_location(self):
+        """Return the URL of the branch we used to be bound to
+        """
+        raise errors.UpgradeRequired(self.base)
+
     def get_commit_builder(self, parents, config=None, timestamp=None, 
                            timezone=None, committer=None, revprops=None, 
                            revision_id=None):
@@ -311,6 +318,10 @@
         """Older format branches cannot bind or unbind."""
         raise errors.UpgradeRequired(self.base)
 
+    def set_append_revisions_only(self, enabled):
+        """Older format branches are never restricted to append-only"""
+        raise errors.UpgradeRequired(self.base)
+
     def last_revision(self):
         """Return last revision id, or None"""
         ph = self.revision_history()
@@ -555,12 +566,18 @@
         result.set_parent(self.bzrdir.root_transport.base)
         return result
 
-    @needs_read_lock
-    def copy_content_into(self, destination, revision_id=None):
-        """Copy the content of self into destination.
-
-        revision_id: if not None, the revision history in the new branch will
-                     be truncated to end with revision_id.
+    def _synchronize_history(self, destination, revision_id):
+        """Synchronize last revision and revision history between branches.
+
+        This version is most efficient when the destination is also a
+        BzrBranch5, but works for BzrBranch6 as long as the revision
+        history is the true lefthand parent history, and all of the revisions
+        are in the destination's repository.  If not, set_revision_history
+        will fail.
+
+        :param destination: The branch to copy the history into
+        :param revision_id: The revision-id to truncate history at.  May
+          be None to copy complete history.
         """
         new_history = self.revision_history()
         if revision_id is not None:
@@ -571,6 +588,15 @@
                 rev = self.repository.get_revision(revision_id)
                 new_history = rev.get_history(self.repository)[1:]
         destination.set_revision_history(new_history)
+
+    @needs_read_lock
+    def copy_content_into(self, destination, revision_id=None):
+        """Copy the content of self into destination.
+
+        revision_id: if not None, the revision history in the new branch will
+                     be truncated to end with revision_id.
+        """
+        self._synchronize_history(destination, revision_id)
         try:
             parent = self.get_parent()
         except errors.InaccessibleParent, e:
@@ -619,6 +645,7 @@
             format.repository_format = weaverepo.RepositoryFormat7()
         else:
             format = self.repository.bzrdir.cloning_metadir()
+            format.branch_format = self._format
         return format
 
     def create_checkout(self, to_location, revision_id=None,
@@ -700,6 +727,38 @@
         """Return the short format description for this format."""
         raise NotImplementedError(self.get_format_description)
 
+    def _initialize_helper(self, a_bzrdir, utf8_files, lock_type='metadir',
+                           set_format=True):
+        """Initialize a branch in a bzrdir, with specified files
+
+        :param a_bzrdir: The bzrdir to initialize the branch in
+        :param utf8_files: The files to create as a list of
+            (filename, content) tuples
+        :param set_format: If True, set the format with
+            self.get_format_string.  (BzrBranch4 has its format set
+            elsewhere)
+        :return: a branch in this format
+        """
+        mutter('creating branch %r in %s', self, a_bzrdir.transport.base)
+        branch_transport = a_bzrdir.get_branch_transport(self)
+        lock_map = {
+            'metadir': ('lock', lockdir.LockDir),
+            'branch4': ('branch-lock', lockable_files.TransportLock),
+        }
+        lock_name, lock_class = lock_map[lock_type]
+        control_files = lockable_files.LockableFiles(branch_transport,
+            lock_name, lock_class)
+        control_files.create_lock()
+        control_files.lock_write()
+        if set_format:
+            control_files.put_utf8('format', self.get_format_string())
+        try:
+            for file, content in utf8_files:
+                control_files.put_utf8(file, content)
+        finally:
+            control_files.unlock()
+        return self.open(a_bzrdir, _found=True)
+
     def initialize(self, a_bzrdir):
         """Create a branch of this format in a_bzrdir."""
         raise NotImplementedError(self.initialize)
@@ -819,21 +878,11 @@
 
     def initialize(self, a_bzrdir):
         """Create a branch of this format in a_bzrdir."""
-        mutter('creating branch in %s', a_bzrdir.transport.base)
-        branch_transport = a_bzrdir.get_branch_transport(self)
         utf8_files = [('revision-history', ''),
                       ('branch-name', ''),
                       ]
-        control_files = lockable_files.LockableFiles(branch_transport,
-                             'branch-lock', lockable_files.TransportLock)
-        control_files.create_lock()
-        control_files.lock_write()
-        try:
-            for file, content in utf8_files:
-                control_files.put_utf8(file, content)
-        finally:
-            control_files.unlock()
-        return self.open(a_bzrdir, _found=True)
+        return self._initialize_helper(a_bzrdir, utf8_files,
+                                       lock_type='branch4', set_format=False)
 
     def __init__(self):
         super(BzrBranchFormat4, self).__init__()
@@ -880,22 +929,10 @@
         
     def initialize(self, a_bzrdir):
         """Create a branch of this format in a_bzrdir."""
-        mutter('creating branch %r in %s', self, a_bzrdir.transport.base)
-        branch_transport = a_bzrdir.get_branch_transport(self)
         utf8_files = [('revision-history', ''),
                       ('branch-name', ''),
                       ]
-        control_files = lockable_files.LockableFiles(branch_transport, 'lock',
-                                                     lockdir.LockDir)
-        control_files.create_lock()
-        control_files.lock_write()
-        control_files.put_utf8('format', self.get_format_string())
-        try:
-            for file, content in utf8_files:
-                control_files.put_utf8(file, content)
-        finally:
-            control_files.unlock()
-        return self.open(a_bzrdir, _found=True, )
+        return self._initialize_helper(a_bzrdir, utf8_files)
 
     def __init__(self):
         super(BzrBranchFormat5, self).__init__()
@@ -922,6 +959,50 @@
         return "Bazaar-NG Metadir branch format 5"
 
 
+class BzrBranchFormat6(BzrBranchFormat5):
+    """Branch format with last-revision
+
+    Unlike previous formats, this has no explicit revision history. Instead,
+    this just stores the last-revision, and the left-hand history leading
+    up to there is the history.
+
+    This format was introduced in bzr 0.15
+    """
+
+    def get_format_string(self):
+        """See BranchFormat.get_format_string()."""
+        return "Bazaar-NG branch format 6\n"
+
+    def get_format_description(self):
+        """See BranchFormat.get_format_description()."""
+        return "Branch format 6"
+
+    def initialize(self, a_bzrdir):
+        """Create a branch of this format in a_bzrdir."""
+        utf8_files = [('last-revision', '0 null:\n'),
+                      ('branch-name', ''),
+                      ('branch.conf', '')
+                      ]
+        return self._initialize_helper(a_bzrdir, utf8_files)
+
+    def open(self, a_bzrdir, _found=False):
+        """Return the branch object for a_bzrdir
+
+        _found is a private parameter, do not use it. It is used to indicate
+               if format probing has already be done.
+        """
+        if not _found:
+            format = BranchFormat.find_format(a_bzrdir)
+            assert format.__class__ == self.__class__
+        transport = a_bzrdir.get_branch_transport(None)
+        control_files = lockable_files.LockableFiles(transport, 'lock',
+                                                     lockdir.LockDir)
+        return BzrBranch6(_format=self,
+                          _control_files=control_files,
+                          a_bzrdir=a_bzrdir,
+                          _repository=a_bzrdir.find_repository())
+
+
 class BranchReferenceFormat(BranchFormat):
     """Bzr branch reference format.
 
@@ -997,6 +1078,7 @@
 __default_format = BzrBranchFormat5()
 BranchFormat.register_format(__default_format)
 BranchFormat.register_format(BranchReferenceFormat())
+BranchFormat.register_format(BzrBranchFormat6())
 BranchFormat.set_default_format(__default_format)
 _legacy_formats = [BzrBranchFormat4(),
                    ]
@@ -1172,12 +1254,19 @@
         rev_history.extend(revision_ids)
         self.set_revision_history(rev_history)
 
+    def _write_revision_history(self, history):
+        """Factored out of set_revision_history.
+
+        This performs the actual writing to disk.
+        It is intended to be called by BzrBranch5.set_revision_history."""
+        self.control_files.put_bytes(
+            'revision-history', '\n'.join(history))
+
     @needs_write_lock
     def set_revision_history(self, rev_history):
         """See Branch.set_revision_history."""
         rev_history = [osutils.safe_revision_id(r) for r in rev_history]
-        self.control_files.put_bytes(
-            'revision-history', '\n'.join(rev_history))
+        self._write_revision_history(rev_history)
         transaction = self.get_transaction()
         history = transaction.map.find_revision_history()
         if history is not None:
@@ -1194,6 +1283,18 @@
         for hook in Branch.hooks['set_rh']:
             hook(self, rev_history)
 
+    @needs_write_lock
+    def set_last_revision_info(self, revno, revision_id):
+        history = self._lefthand_history(revision_id)
+        assert len(history) == revno, '%d != %d' % (len(history), revno)
+        self.set_revision_history(history)
+
+    def _gen_revision_history(self):
+        get_cached_utf8 = cache_utf8.get_cached_utf8
+        history = [get_cached_utf8(l.rstrip('\r\n')) for l in
+                self.control_files.get('revision-history').readlines()]
+        return history
+
     @needs_read_lock
     def revision_history(self):
         """See Branch.revision_history."""
@@ -1202,27 +1303,15 @@
         if history is not None:
             # mutter("cache hit for revision-history in %s", self)
             return list(history)
-        get_cached_utf8 = cache_utf8.get_cached_utf8
-        history = [get_cached_utf8(l.rstrip('\r\n')) for l in
-                self.control_files.get('revision-history').readlines()]
+        history = self._gen_revision_history()
         transaction.map.add_revision_history(history)
         # this call is disabled because revision_history is 
         # not really an object yet, and the transaction is for objects.
         # transaction.register_clean(history, precious=True)
         return list(history)
 
-    @needs_write_lock
-    def generate_revision_history(self, revision_id, last_rev=None, 
-        other_branch=None):
-        """Create a new revision history that will finish with revision_id.
-        
-        :param revision_id: the new tip to use.
-        :param last_rev: The previous last_revision. If not None, then this
-            must be a ancestory of revision_id, or DivergedBranches is raised.
-        :param other_branch: The other branch that DivergedBranches should
-            raise with respect to.
-        """
-        revision_id = osutils.safe_revision_id(revision_id)
+    def _lefthand_history(self, revision_id, last_rev=None,
+                          other_branch=None):
         # stop_revision must be a descendant of last_revision
         stop_graph = self.repository.get_revision_graph(revision_id)
         if last_rev is not None and last_rev not in stop_graph:
@@ -1239,7 +1328,22 @@
             except IndexError:
                 current_rev_id = None
         new_history.reverse()
-        self.set_revision_history(new_history)
+        return new_history
+
+    @needs_write_lock
+    def generate_revision_history(self, revision_id, last_rev=None,
+        other_branch=None):
+        """Create a new revision history that will finish with revision_id.
+
+        :param revision_id: the new tip to use.
+        :param last_rev: The previous last_revision. If not None, then this
+            must be a ancestory of revision_id, or DivergedBranches is raised.
+        :param other_branch: The other branch that DivergedBranches should
+            raise with respect to.
+        """
+        revision_id = osutils.safe_revision_id(revision_id)
+        self.set_revision_history(self._lefthand_history(revision_id,
+            last_rev, other_branch))
 
     @needs_write_lock
     def update_revisions(self, other, stop_revision=None):
@@ -1316,6 +1420,15 @@
         finally:
             source.unlock()
 
+    def _get_parent_location(self):
+        _locs = ['parent', 'pull', 'x-pull']
+        for l in _locs:
+            try:
+                return self.control_files.get(l).read().strip('\n')
+            except NoSuchFile:
+                pass
+        return None
+
     @needs_read_lock
     def push(self, target, overwrite=False, stop_revision=None,
         _hook_master=None, _run_hooks=True):
@@ -1353,22 +1466,18 @@
     def get_parent(self):
         """See Branch.get_parent."""
 
-        _locs = ['parent', 'pull', 'x-pull']
         assert self.base[-1] == '/'
-        for l in _locs:
-            try:
-                parent = self.control_files.get(l).read().strip('\n')
-            except NoSuchFile:
-                continue
-            # This is an old-format absolute path to a local branch
-            # turn it into a url
-            if parent.startswith('/'):
-                parent = urlutils.local_path_to_url(parent.decode('utf8'))
-            try:
-                return urlutils.join(self.base[:-1], parent)
-            except errors.InvalidURLJoin, e:
-                raise errors.InaccessibleParent(parent, self.base)
-        return None
+        parent = self._get_parent_location()
+        if parent is None:
+            return parent
+        # This is an old-format absolute path to a local branch
+        # turn it into a url
+        if parent.startswith('/'):
+            parent = urlutils.local_path_to_url(parent.decode('utf8'))
+        try:
+            return urlutils.join(self.base[:-1], parent)
+        except errors.InvalidURLJoin, e:
+            raise errors.InaccessibleParent(parent, self.base)
 
     def get_push_location(self):
         """See Branch.get_push_location."""
@@ -1389,9 +1498,7 @@
         # FIXUP this and get_parent in a future branch format bump:
         # read and rewrite the file, and have the new format code read
         # using .get not .get_utf8. RBC 20060125
-        if url is None:
-            self.control_files._transport.delete('parent')
-        else:
+        if url is not None:
             if isinstance(url, unicode):
                 try: 
                     url = url.encode('ascii')
@@ -1399,8 +1506,14 @@
                     raise bzrlib.errors.InvalidURL(url,
                         "Urls must be 7-bit ascii, "
                         "use bzrlib.urlutils.escape")
-                    
             url = urlutils.relative_url(self.base, url)
+        self._set_parent_location(url)
+
+    def _set_parent_location(self, url):
+        if url is None:
+            self.control_files._transport.delete('parent')
+        else:
+            assert isinstance(url, str)
             self.control_files.put_bytes('parent', url + '\n')
 
     @deprecated_function(zero_nine)
@@ -1580,6 +1693,192 @@
         return None
 
 
+class BzrBranch6(BzrBranch5):
+
+    @needs_read_lock
+    def last_revision_info(self):
+        revision_string = self.control_files.get_utf8('last-revision').read()
+        revno, revision_id = revision_string.rstrip('\n').split(' ', 1)
+        revno = int(revno)
+        return revno, revision_id
+
+    def last_revision(self):
+        """Return last revision id, or None"""
+        revision_id = self.last_revision_info()[1]
+        if revision_id == _mod_revision.NULL_REVISION:
+            revision_id = None
+        return revision_id
+
+    def _write_last_revision_info(self, revno, revision_id):
+        """Simply write out the revision id, with no checks.
+
+        Use set_last_revision_info to perform this safely.
+
+        Does not update the revision_history cache.
+        Intended to be called by set_last_revision_info and
+        _write_revision_history.
+        """
+        if revision_id is None:
+            revision_id = 'null:'
+        out_string = '%d %s\n' % (revno, revision_id)
+        self.control_files.put_bytes('last-revision', out_string)
+
+    @needs_write_lock
+    def set_last_revision_info(self, revno, revision_id):
+        if self._get_append_revisions_only():
+            self._check_history_violation(revision_id)
+        self._write_last_revision_info(revno, revision_id)
+        transaction = self.get_transaction()
+        cached_history = transaction.map.find_revision_history()
+        if cached_history is not None:
+            transaction.map.remove_object(cached_history)
+
+    def _check_history_violation(self, revision_id):
+        last_revision = self.last_revision()
+        if last_revision is None:
+            return
+        if last_revision not in self._lefthand_history(revision_id):
+            raise errors.AppendRevisionsOnlyViolation(self.base)
+
+    def _gen_revision_history(self):
+        """Generate the revision history from last revision
+        """
+        history = list(self.repository.iter_reverse_revision_history(
+            self.last_revision()))
+        history.reverse()
+        return history
+
+    def _write_revision_history(self, history):
+        """Factored out of set_revision_history.
+
+        This performs the actual writing to disk, with format-specific checks.
+        It is intended to be called by BzrBranch5.set_revision_history.
+        """
+        if len(history) == 0:
+            last_revision = 'null:'
+        else:
+            if history != self._lefthand_history(history[-1]):
+                raise errors.NotLefthandHistory(history)
+            last_revision = history[-1]
+        if self._get_append_revisions_only():
+            self._check_history_violation(last_revision)
+        self._write_last_revision_info(len(history), last_revision)
+
+    @needs_write_lock
+    def append_revision(self, *revision_ids):
+        if len(revision_ids) == 0:
+            return
+        prev_revno, prev_revision = self.last_revision_info()
+        for revision in self.repository.get_revisions(revision_ids):
+            if prev_revision == _mod_revision.NULL_REVISION:
+                if revision.parent_ids != []:
+                    raise errors.NotLeftParentDescendant(self, prev_revision,
+                                                         revision.revision_id)
+            else:
+                if revision.parent_ids[0] != prev_revision:
+                    raise errors.NotLeftParentDescendant(self, prev_revision,
+                                                         revision.revision_id)
+            prev_revision = revision.revision_id
+        self.set_last_revision_info(prev_revno + len(revision_ids),
+                                    revision_ids[-1])
+
+    def _set_config_location(self, name, url, config=None,
+                             make_relative=False):
+        if config is None:
+            config = self.get_config()
+        if url is None:
+            url = ''
+        elif make_relative:
+            url = urlutils.relative_url(self.base, url)
+        config.set_user_option(name, url)
+
+
+    def _get_config_location(self, name, config=None):
+        if config is None:
+            config = self.get_config()
+        location = config.get_user_option(name)
+        if location == '':
+            location = None
+        return location
+
+    @needs_write_lock
+    def _set_parent_location(self, url):
+        """Set the parent branch"""
+        self._set_config_location('parent_location', url, make_relative=True)
+
+    @needs_read_lock
+    def _get_parent_location(self):
+        """Set the parent branch"""
+        return self._get_config_location('parent_location')
+
+    def set_push_location(self, location):
+        """See Branch.set_push_location."""
+        self._set_config_location('push_location', location)
+
+    def set_bound_location(self, location):
+        """See Branch.set_push_location."""
+        result = None
+        config = self.get_config()
+        if location is None:
+            if config.get_user_option('bound') != 'True':
+                return False
+            else:
+                config.set_user_option('bound', 'False')
+                return True
+        else:
+            self._set_config_location('bound_location', location,
+                                      config=config)
+            config.set_user_option('bound', 'True')
+        return True
+
+    def _get_bound_location(self, bound):
+        """Return the bound location in the config file.
+
+        Return None if the bound parameter does not match"""
+        config = self.get_config()
+        config_bound = (config.get_user_option('bound') == 'True')
+        if config_bound != bound:
+            return None
+        return self._get_config_location('bound_location', config=config)
+
+    def get_bound_location(self):
+        """See Branch.set_push_location."""
+        return self._get_bound_location(True)
+
+    def get_old_bound_location(self):
+        """See Branch.get_old_bound_location"""
+        return self._get_bound_location(False)
+
+    def set_append_revisions_only(self, enabled):
+        if enabled:
+            value = 'True'
+        else:
+            value = 'False'
+        self.get_config().set_user_option('append_revisions_only', value)
+
+    def _get_append_revisions_only(self):
+        value = self.get_config().get_user_option('append_revisions_only')
+        return value == 'True'
+
+    def _synchronize_history(self, destination, revision_id):
+        """Synchronize last revision and revision history between branches.
+
+        This version is most efficient when the destination is also a
+        BzrBranch6, but works for BzrBranch5, as long as the destination's
+        repository contains all the lefthand ancestors of the intended
+        last_revision.  If not, set_last_revision_info will fail.
+
+        :param destination: The branch to copy the history into
+        :param revision_id: The revision-id to truncate history at.  May
+          be None to copy complete history.
+        """
+        if revision_id is None:
+            revno, revision_id = self.last_revision_info()
+        else:
+            revno = self.revision_id_to_revno(revision_id)
+        destination.set_last_revision_info(revno, revision_id)
+
+
 class BranchTestProviderAdapter(object):
     """A tool to generate a suite testing multiple branch formats at once.
 
@@ -1639,3 +1938,30 @@
     """See bzrlib.workingtree.is_control_file."""
     from bzrlib import workingtree
     return workingtree.is_control_file(*args, **kwargs)
+
+
+class Converter5to6(object):
+    """Perform an in-place upgrade of format 5 to format 6"""
+
+    def convert(self, branch):
+        # Data for 5 and 6 can peacefully coexist.
+        format = BzrBranchFormat6()
+        new_branch = format.open(branch.bzrdir, _found=True)
+
+        # Copy source data into target
+        new_branch.set_last_revision_info(*branch.last_revision_info())
+        new_branch.set_parent(branch.get_parent())
+        new_branch.set_bound_location(branch.get_bound_location())
+        new_branch.set_push_location(branch.get_push_location())
+
+        # Copying done; now update target format
+        new_branch.control_files.put_utf8('format',
+            format.get_format_string())
+
+        # Clean up old files
+        new_branch.control_files._transport.delete('revision-history')
+        try:
+            branch.set_parent(None)
+        except NoSuchFile:
+            pass
+        branch.set_bound_location(None)

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-02-13 11:48:14 +0000
+++ bzrlib/builtins.py	2007-02-15 18:01:21 +0000
@@ -237,8 +237,6 @@
     this will refuse to run against one.
     """
 
-    hidden = True
-
     takes_args = ['location?']
 
     def run(self, location='.'):
@@ -537,18 +535,29 @@
     location can be accessed.
     """
 
-    takes_options = ['remember', 'overwrite', 'revision', 'verbose']
+    takes_options = ['remember', 'overwrite', 'revision', 'verbose',
+        Option('directory',
+            help='branch to pull into, '
+                 'rather than the one containing the working directory',
+            short_name='d',
+            type=unicode,
+            ),
+        ]
     takes_args = ['location?']
     encoding_type = 'replace'
 
-    def run(self, location=None, remember=False, overwrite=False, revision=None, verbose=False):
+    def run(self, location=None, remember=False, overwrite=False,
+            revision=None, verbose=False,
+            directory=None):
         # FIXME: too much stuff is in the command class
+        if directory is None:
+            directory = u'.'
         try:
-            tree_to = WorkingTree.open_containing(u'.')[0]
+            tree_to = WorkingTree.open_containing(directory)[0]
             branch_to = tree_to.branch
         except errors.NoWorkingTree:
             tree_to = None
-            branch_to = Branch.open_containing(u'.')[0]
+            branch_to = Branch.open_containing(directory)[0]
 
         reader = None
         if location is not None:
@@ -568,7 +577,6 @@
                 self.outf.write("Using saved location: %s\n" % display_url)
                 location = stored_loc
 
-
         if reader is not None:
             install_bundle(branch_to.repository, reader)
             branch_from = branch_to
@@ -589,7 +597,8 @@
 
         old_rh = branch_to.revision_history()
         if tree_to is not None:
-            count = tree_to.pull(branch_from, overwrite, rev_id)
+            count = tree_to.pull(branch_from, overwrite, rev_id,
+                delta.ChangeReporter(tree_to.inventory))
         else:
             count = branch_to.pull(branch_from, overwrite, rev_id)
         note('%d revision(s) pulled.' % (count,))
@@ -630,24 +639,33 @@
     """
 
     takes_options = ['remember', 'overwrite', 'verbose',
-                     Option('create-prefix',
-                            help='Create the path leading up to the branch '
-                                 'if it does not already exist'),
-                     Option('use-existing-dir',
-                            help='By default push will fail if the target'
-                                 ' directory exists, but does not already'
-                                 ' have a control directory. This flag will'
-                                 ' allow push to proceed.'),
-                     ]
+        Option('create-prefix',
+               help='Create the path leading up to the branch '
+                    'if it does not already exist'),
+        Option('directory',
+            help='branch to push from, '
+                 'rather than the one containing the working directory',
+            short_name='d',
+            type=unicode,
+            ),
+        Option('use-existing-dir',
+               help='By default push will fail if the target'
+                    ' directory exists, but does not already'
+                    ' have a control directory. This flag will'
+                    ' allow push to proceed.'),
+        ]
     takes_args = ['location?']
     encoding_type = 'replace'
 
     def run(self, location=None, remember=False, overwrite=False,
-            create_prefix=False, verbose=False, use_existing_dir=False):
+            create_prefix=False, verbose=False,
+            use_existing_dir=False,
+            directory=None):
         # FIXME: Way too big!  Put this into a function called from the
         # command.
-        
-        br_from = Branch.open_containing('.')[0]
+        if directory is None:
+            directory = '.'
+        br_from = Branch.open_containing(directory)[0]
         stored_loc = br_from.get_push_location()
         if location is None:
             if stored_loc is None:
@@ -813,14 +831,8 @@
         elif len(revision) > 1:
             raise errors.BzrCommandError(
                 'bzr branch --revision takes exactly 1 revision value')
-        try:
-            br_from = Branch.open(from_location)
-        except OSError, e:
-            if e.errno == errno.ENOENT:
-                raise errors.BzrCommandError('Source location "%s" does not'
-                                             ' exist.' % to_location)
-            else:
-                raise
+
+        br_from = Branch.open(from_location)
         br_from.lock_read()
         try:
             if basis is not None:
@@ -1194,8 +1206,11 @@
                 value_switches=True,
                 title="Branch Format",
                 ),
+         Option('append-revisions-only',
+                help='Never change revnos or the existing log.'
+                '  Append revisions to it only.')
          ]
-    def run(self, location=None, format=None):
+    def run(self, location=None, format=None, append_revisions_only=False):
         if format is None:
             format = bzrdir.format_registry.make_bzrdir('default')
         if location is None:
@@ -1218,7 +1233,8 @@
             existing_bzrdir = bzrdir.BzrDir.open(location)
         except errors.NotBranchError:
             # really a NotBzrDir error...
-            bzrdir.BzrDir.create_branch_convenience(location, format=format)
+            branch = bzrdir.BzrDir.create_branch_convenience(location,
+                                                             format=format)
         else:
             from bzrlib.transport.local import LocalTransport
             if existing_bzrdir.has_branch():
@@ -1227,8 +1243,14 @@
                         raise errors.BranchExistsWithoutWorkingTree(location)
                 raise errors.AlreadyBranchError(location)
             else:
-                existing_bzrdir.create_branch()
+                branch = existing_bzrdir.create_branch()
                 existing_bzrdir.create_workingtree()
+        if append_revisions_only:
+            try:
+                branch.set_append_revisions_only(True)
+            except errors.UpgradeRequired:
+                raise errors.BzrCommandError('This branch format cannot be set'
+                    ' to append-revisions-only.  Try --experimental-branch6')
 
 
 class cmd_init_repository(Command):
@@ -1533,52 +1555,58 @@
             dir, relpath = bzrdir.BzrDir.open_containing(location)
             b = dir.open_branch()
 
-        if revision is None:
-            rev1 = None
-            rev2 = None
-        elif len(revision) == 1:
-            rev1 = rev2 = revision[0].in_history(b).revno
-        elif len(revision) == 2:
-            if revision[1].get_branch() != revision[0].get_branch():
-                # b is taken from revision[0].get_branch(), and
-                # show_log will use its revision_history. Having
-                # different branches will lead to weird behaviors.
+        b.lock_read()
+        try:
+            if revision is None:
+                rev1 = None
+                rev2 = None
+            elif len(revision) == 1:
+                rev1 = rev2 = revision[0].in_history(b).revno
+            elif len(revision) == 2:
+                if revision[1].get_branch() != revision[0].get_branch():
+                    # b is taken from revision[0].get_branch(), and
+                    # show_log will use its revision_history. Having
+                    # different branches will lead to weird behaviors.
+                    raise errors.BzrCommandError(
+                        "Log doesn't accept two revisions in different"
+                        " branches.")
+                if revision[0].spec is None:
+                    # missing begin-range means first revision
+                    rev1 = 1
+                else:
+                    rev1 = revision[0].in_history(b).revno
+
+                if revision[1].spec is None:
+                    # missing end-range means last known revision
+                    rev2 = b.revno()
+                else:
+                    rev2 = revision[1].in_history(b).revno
+            else:
                 raise errors.BzrCommandError(
-                    "Log doesn't accept two revisions in different branches.")
-            if revision[0].spec is None:
-                # missing begin-range means first revision
-                rev1 = 1
-            else:
-                rev1 = revision[0].in_history(b).revno
-
-            if revision[1].spec is None:
-                # missing end-range means last known revision
-                rev2 = b.revno()
-            else:
-                rev2 = revision[1].in_history(b).revno
-        else:
-            raise errors.BzrCommandError('bzr log --revision takes one or two values.')
-
-        # By this point, the revision numbers are converted to the +ve
-        # form if they were supplied in the -ve form, so we can do
-        # this comparison in relative safety
-        if rev1 > rev2:
-            (rev2, rev1) = (rev1, rev2)
-
-        if log_format is None:
-            log_format = log.log_formatter_registry.get_default(b)
-
-        lf = log_format(show_ids=show_ids, to_file=self.outf,
-                        show_timezone=timezone)
-
-        show_log(b,
-                 lf,
-                 file_id,
-                 verbose=verbose,
-                 direction=direction,
-                 start_revision=rev1,
-                 end_revision=rev2,
-                 search=message)
+                    'bzr log --revision takes one or two values.')
+
+            # By this point, the revision numbers are converted to the +ve
+            # form if they were supplied in the -ve form, so we can do
+            # this comparison in relative safety
+            if rev1 > rev2:
+                (rev2, rev1) = (rev1, rev2)
+
+            if log_format is None:
+                log_format = log.log_formatter_registry.get_default(b)
+
+            lf = log_format(show_ids=show_ids, to_file=self.outf,
+                            show_timezone=timezone)
+
+            show_log(b,
+                     lf,
+                     file_id,
+                     verbose=verbose,
+                     direction=direction,
+                     start_revision=rev1,
+                     end_revision=rev2,
+                     search=message)
+        finally:
+            b.unlock()
 
 
 def get_log_format(long=False, short=False, line=False, default='long'):
@@ -2357,6 +2385,10 @@
     default, use --remember. The value will only be saved if the remote
     location can be accessed.
 
+    The results of the merge are placed into the destination working
+    directory, where they can be reviewed (with bzr diff), tested, and then
+    committed to record the result of the merge.
+
     Examples:
 
     To merge the latest revision from bzr.dev
@@ -2375,23 +2407,33 @@
     """
     takes_args = ['branch?']
     takes_options = ['revision', 'force', 'merge-type', 'reprocess', 'remember',
-                     Option('show-base', help="Show base revision text in "
-                            "conflicts"),
-                     Option('uncommitted', help='Apply uncommitted changes'
-                            ' from a working copy, instead of branch changes'),
-                     Option('pull', help='If the destination is already'
-                             ' completely merged into the source, pull from the'
-                             ' source rather than merging. When this happens,'
-                             ' you do not need to commit the result.'),
-                     ]
+        Option('show-base', help="Show base revision text in "
+               "conflicts"),
+        Option('uncommitted', help='Apply uncommitted changes'
+               ' from a working copy, instead of branch changes'),
+        Option('pull', help='If the destination is already'
+                ' completely merged into the source, pull from the'
+                ' source rather than merging. When this happens,'
+                ' you do not need to commit the result.'),
+        Option('directory',
+            help='branch to merge into, '
+                 'rather than the one containing the working directory',
+            short_name='d',
+            type=unicode,
+            ),
+    ]
 
     def run(self, branch=None, revision=None, force=False, merge_type=None,
-            show_base=False, reprocess=False, remember=False, 
-            uncommitted=False, pull=False):
+            show_base=False, reprocess=False, remember=False,
+            uncommitted=False, pull=False,
+            directory=None,
+            ):
         if merge_type is None:
             merge_type = _mod_merge.Merge3Merger
 
-        tree = WorkingTree.open_containing(u'.')[0]
+        if directory is None: directory = u'.'
+        tree = WorkingTree.open_containing(directory)[0]
+        change_reporter = delta.ChangeReporter(tree.inventory)
 
         if branch is not None:
             try:
@@ -2400,7 +2442,7 @@
                 pass # Continue on considering this url a Branch
             else:
                 conflicts = merge_bundle(reader, tree, not force, merge_type,
-                                            reprocess, show_base)
+                                         reprocess, show_base, change_reporter)
                 if conflicts == 0:
                     return 0
                 else:
@@ -2459,7 +2501,9 @@
                     reprocess=reprocess,
                     show_base=show_base,
                     pull=pull,
-                    pb=pb, file_list=interesting_files)
+                    this_dir=directory,
+                    pb=pb, file_list=interesting_files,
+                    change_reporter=change_reporter)
             finally:
                 pb.finished()
             if conflict_count != 0:
@@ -2913,11 +2957,21 @@
     See "help checkouts" for more information on checkouts.
     """
 
-    takes_args = ['location']
+    takes_args = ['location?']
     takes_options = []
 
     def run(self, location=None):
         b, relpath = Branch.open_containing(u'.')
+        if location is None:
+            try:
+                location = b.get_old_bound_location()
+            except errors.UpgradeRequired:
+                raise errors.BzrCommandError('No location supplied.  '
+                    'This format does not remember old locations.')
+            else:
+                if location is None:
+                    raise errors.BzrCommandError('No location supplied and no '
+                        'previous location known')
         b_other = Branch.open(location)
         try:
             b.bind(b_other)
@@ -3117,7 +3171,8 @@
                   merge_type=None,
                   file_list=None, show_base=False, reprocess=False,
                   pull=False,
-                  pb=DummyProgress()):
+                  pb=DummyProgress(),
+                  change_reporter=None):
     """Merge changes into a tree.
 
     base_revision
@@ -3161,7 +3216,7 @@
         raise errors.BzrCommandError("Cannot do conflict reduction and show base.")
     try:
         merger = _mod_merge.Merger(this_tree.branch, this_tree=this_tree,
-                                   pb=pb)
+                                   pb=pb, change_reporter=change_reporter)
         merger.pp = ProgressPhase("Merge phase", 5, pb)
         merger.pp.next_phase()
         merger.check_basis(check_clean)

=== modified file 'bzrlib/bundle/apply_bundle.py'
--- bzrlib/bundle/apply_bundle.py	2006-10-05 05:37:25 +0000
+++ bzrlib/bundle/apply_bundle.py	2007-02-09 15:56:49 +0000
@@ -42,14 +42,15 @@
 
 
 def merge_bundle(reader, tree, check_clean, merge_type, 
-                    reprocess, show_base):
+                    reprocess, show_base, change_reporter=None):
     """Merge a revision bundle into the current tree."""
     pb = bzrlib.ui.ui_factory.nested_progress_bar()
     try:
         pp = ProgressPhase("Merge phase", 6, pb)
         pp.next_phase()
         install_bundle(tree.branch.repository, reader)
-        merger = Merger(tree.branch, this_tree=tree, pb=pb)
+        merger = Merger(tree.branch, this_tree=tree, pb=pb,
+                        change_reporter=change_reporter)
         merger.pp = pp
         merger.pp.next_phase()
         merger.check_basis(check_clean, require_commits=False)

=== modified file 'bzrlib/bundle/serializer/__init__.py'
--- bzrlib/bundle/serializer/__init__.py	2006-10-05 05:37:25 +0000
+++ bzrlib/bundle/serializer/__init__.py	2007-02-09 01:15:10 +0000
@@ -298,5 +298,5 @@
 
 register_lazy('0.8', 'bzrlib.bundle.serializer.v08', 'BundleSerializerV08')
 register_lazy('0.9', 'bzrlib.bundle.serializer.v09', 'BundleSerializerV09')
-register_lazy(None, 'bzrlib.bundle.serializer.v08', 'BundleSerializerV08')
+register_lazy(None, 'bzrlib.bundle.serializer.v09', 'BundleSerializerV09')
 

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2007-02-13 05:22:39 +0000
+++ bzrlib/bzrdir.py	2007-02-15 14:08:23 +0000
@@ -925,8 +925,7 @@
 
     def create_branch(self):
         """See BzrDir.create_branch."""
-        from bzrlib.branch import BranchFormat
-        return BranchFormat.get_default_format().initialize(self)
+        return self._format.get_branch_format().initialize(self)
 
     def create_repository(self, shared=False):
         """See BzrDir.create_repository."""
@@ -1011,6 +1010,13 @@
                 return True
         except errors.NoRepositoryPresent:
             pass
+        try:
+            if not isinstance(self.open_branch()._format,
+                              format.get_branch_format().__class__):
+                # the repository needs an upgrade.
+                return True
+        except errors.NotBranchError:
+            pass
         # currently there are no other possible conversions for meta1 formats.
         return False
 
@@ -1439,6 +1445,18 @@
 
     _lock_class = lockdir.LockDir
 
+    def __init__(self):
+        self._branch_format = None
+
+    def get_branch_format(self):
+        if self._branch_format is None:
+            from bzrlib.branch import BranchFormat
+            self._branch_format = BranchFormat.get_default_format()
+        return self._branch_format
+
+    def set_branch_format(self, format):
+        self._branch_format = format
+
     def get_converter(self, format=None):
         """See BzrDirFormat.get_converter()."""
         if format is None:
@@ -1970,6 +1988,18 @@
                 self.pb.note('starting repository conversion')
                 converter = CopyConverter(self.target_format.repository_format)
                 converter.convert(repo, pb)
+        try:
+            branch = self.bzrdir.open_branch()
+        except errors.NotBranchError:
+            pass
+        else:
+            # Avoid circular imports
+            from bzrlib import branch as _mod_branch
+            if (branch._format.__class__ is _mod_branch.BzrBranchFormat5 and
+                self.target_format.get_branch_format().__class__ is
+                _mod_branch.BzrBranchFormat6):
+                branch_converter = _mod_branch.Converter5to6()
+                branch_converter.convert(branch)
         return to_convert
 
 
@@ -1988,28 +2018,35 @@
     """
 
     def register_metadir(self, key, repo, help, native=True, deprecated=False,
-            repo_module='bzrlib.repository'):
+                         branch_format=None):
         """Register a metadir subformat.
 
         These all use a BzrDirMetaFormat1 bzrdir, but can be parameterized
         by the Repository format.
 
-        :param repo: The repository format class name as a string.
-
-        :param repo_module: The module from which the repository class 
-        should be lazily loaded.  By default this is bzrlib.repository.
+        :param repo: The fully-qualified repository format class name as a
+        string.
         """
         # This should be expanded to support setting WorkingTree and Branch
         # formats, once BzrDirMetaFormat1 supports that.
         def helper():
-            mod = __import__(repo_module, globals(), locals(), [repo])
-            try:
-                repo_format_class = getattr(mod, repo)
+            import bzrlib.branch
+            mod_name, repo_factory_name = repo.rsplit('.', 1)
+            try:
+                mod = __import__(mod_name, globals(), locals(),
+                        [repo_factory_name])
+            except ImportError, e:
+                raise ImportError('failed to load repository %s: %s'
+                    % (repo, e))
+            try:
+                repo_format_class = getattr(mod, repo_factory_name)
             except AttributeError:
                 raise AttributeError('no repository format %r in module %r' 
                     % (repo, mod))
             bd = BzrDirMetaFormat1()
             bd.repository_format = repo_format_class()
+            if branch_format is not None:
+                bd.set_branch_format(getattr(bzrlib.branch, branch_format)())
             return bd
         self.register(key, helper, help, native, deprecated)
 
@@ -2103,15 +2140,20 @@
     ' support checkouts or shared repositories.',
     deprecated=True)
 format_registry.register_metadir('knit',
-    'RepositoryFormatKnit1',
+    'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
     'Format using knits.  Recommended.',
-    repo_module='bzrlib.repofmt.knitrepo')
+    branch_format='BzrBranchFormat5')
 format_registry.set_default('knit')
-format_registry.register_metadir('metaweave', 'RepositoryFormat7',
+format_registry.register_metadir('metaweave',
+    'bzrlib.repofmt.weaverepo.RepositoryFormat7',
     'Transitional format in 0.8.  Slower than knit.',
     deprecated=True,
-    repo_module='bzrlib.repofmt.weaverepo')
+    )
 format_registry.register_metadir('experimental-knit2',
-    'RepositoryFormatKnit2',
-    'Experimental successor to knit.  Use at your own risk.',
-    repo_module='bzrlib.repofmt.knitrepo')
+    'bzrlib.repofmt.knitrepo.RepositoryFormatKnit2',
+    'Experimental successor to knit.  Use at your own risk.',
+    branch_format='BzrBranchFormat5')
+format_registry.register_metadir('experimental-branch6',
+    'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
+    'Experimental successor to knit.  Use at your own risk.',
+    branch_format='BzrBranchFormat6')

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2007-02-06 23:53:30 +0000
+++ bzrlib/commit.py	2007-02-15 15:11:31 +0000
@@ -254,12 +254,16 @@
                 # this is so that we still consier the master branch
                 # - in a checkout scenario the tree may have no
                 # parents but the branch may do.
-                first_tree_parent = None
-            master_last = self.master_branch.last_revision()
-            if (master_last is not None and
-                master_last != first_tree_parent):
-                raise errors.OutOfDateTree(self.work_tree)
-    
+                first_tree_parent = bzrlib.revision.NULL_REVISION
+            old_revno, master_last = self.master_branch.last_revision_info()
+            if master_last != first_tree_parent:
+                if master_last != bzrlib.revision.NULL_REVISION:
+                    raise errors.OutOfDateTree(self.work_tree)
+            if self.branch.repository.has_revision(first_tree_parent):
+                new_revno = old_revno + 1
+            else:
+                # ghost parents never appear in revision history.
+                new_revno = 1
             if strict:
                 # raise an exception as soon as we find a single unknown.
                 for unknown in self.work_tree.unknowns():
@@ -323,16 +327,17 @@
                 # now the master has the revision data
                 # 'commit' to the master first so a timeout here causes the local
                 # branch to be out of date
-                self.master_branch.append_revision(self.rev_id)
+                self.master_branch.set_last_revision_info(new_revno,
+                                                          self.rev_id)
 
             # and now do the commit locally.
-            self.branch.append_revision(self.rev_id)
+            self.branch.set_last_revision_info(new_revno, self.rev_id)
 
             rev_tree = self.builder.revision_tree()
             self.work_tree.set_parent_trees([(self.rev_id, rev_tree)])
             # now the work tree is up to date with the branch
             
-            self.reporter.completed(self.branch.revno(), self.rev_id)
+            self.reporter.completed(new_revno, self.rev_id)
             # old style commit hooks - should be deprecated ? (obsoleted in
             # 0.15)
             if self.config.post_commit() is not None:
@@ -350,11 +355,9 @@
             else:
                 hook_master = self.master_branch
                 hook_local = self.branch
-            new_revno = self.branch.revno()
             # With bound branches, when the master is behind the local branch,
             # the 'old_revno' and old_revid values here are incorrect.
             # XXX: FIXME ^. RBC 20060206
-            old_revno = new_revno - 1
             if self.parents:
                 old_revid = self.parents[0]
             else:
@@ -521,8 +524,8 @@
         # TODO: Make sure that this list doesn't contain duplicate 
         # entries and the order is preserved when doing this.
         self.parents = self.work_tree.get_parent_ids()
-        self.parent_invs = []
-        for revision in self.parents:
+        self.parent_invs = [self.basis_inv]
+        for revision in self.parents[1:]:
             if self.branch.repository.has_revision(revision):
                 mutter('commit parent revision {%s}', revision)
                 inventory = self.branch.repository.get_inventory(revision)

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2007-02-06 17:12:12 +0000
+++ bzrlib/delta.py	2007-02-08 22:42:45 +0000
@@ -344,8 +344,8 @@
         exe_change = False
         # files are "renamed" if they are moved or if name changes, as long
         # as it had a value
-        if None not in name and (name[0] != name[1] or
-                                 parent_id[0] != parent_id[1]):
+        if None not in name and None not in parent_id and\
+            (name[0] != name[1] or parent_id[0] != parent_id[1]):
             renamed = True
         else:
             renamed = False

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2007-02-03 10:02:11 +0000
+++ bzrlib/errors.py	2007-02-14 18:16:33 +0000
@@ -778,6 +778,19 @@
         BzrError.__init__(self, branch=branch, revision=revision)
 
 
+class NotLeftParentDescendant(BzrError):
+
+    _fmt = "Revision %(old_revision)s is not the left parent of"\
+        " %(new_revision)s, but branch %(branch_location)s expects this"
+
+    internal_error = True
+
+    def __init__(self, branch, old_revision, new_revision):
+        BzrError.__init__(self, branch_location=branch.base,
+                          old_revision=old_revision,
+                          new_revision=new_revision)
+
+
 class NoSuchRevisionSpec(BzrError):
 
     _fmt = "No namespace registered for string: %(spec)r"
@@ -803,6 +816,18 @@
     _fmt = "%(branch)s is missing %(object_type)s {%(object_id)s}"
 
 
+class AppendRevisionsOnlyViolation(BzrError):
+
+    _fmt = 'Operation denied because it would change the main history, '\
+           'which is not permitted by the append_revisions_only setting on'\
+           ' branch "%(location)s".'
+
+    def __init__(self, location):
+       import bzrlib.urlutils as urlutils
+       location = urlutils.unescape_for_display(location, 'ascii')
+       BzrError.__init__(self, location=location)
+
+
 class DivergedBranches(BzrError):
     
     _fmt = "These branches have diverged.  Use the merge command to reconcile them."""
@@ -814,6 +839,16 @@
         self.branch2 = branch2
 
 
+class NotLefthandHistory(BzrError):
+
+    _fmt = "Supplied history does not follow left-hand parents"
+
+    internal_error = True
+
+    def __init__(self, history):
+        BzrError.__init__(self, history=history)
+
+
 class UnrelatedBranches(BzrError):
 
     _fmt = "Branches have no common ancestor, and no merge base revision was specified."

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2007-02-07 16:10:56 +0000
+++ bzrlib/merge.py	2007-02-14 21:57:40 +0000
@@ -88,7 +88,7 @@
 
 class Merger(object):
     def __init__(self, this_branch, other_tree=None, base_tree=None, 
-                 this_tree=None, pb=DummyProgress()):
+                 this_tree=None, pb=DummyProgress(), change_reporter=None):
         object.__init__(self)
         assert this_tree is not None, "this_tree is required"
         self.this_branch = this_branch
@@ -106,7 +106,7 @@
         self.reprocess = False
         self._pb = pb 
         self.pp = None
-
+        self.change_reporter = change_reporter
 
     def revision_tree(self, revision_id):
         return self.this_branch.repository.revision_tree(revision_id)
@@ -269,7 +269,9 @@
         elif self.show_base:
             raise BzrError("Showing base is not supported for this"
                                   " merge type. %s" % self.merge_type)
-        merge = self.merge_type(pb=self._pb, **kwargs)
+        merge = self.merge_type(pb=self._pb,
+                                change_reporter=self.change_reporter,
+                                **kwargs)
         if len(merge.cooked_conflicts) == 0:
             if not self.ignore_zero:
                 note("All changes applied successfully.")
@@ -357,7 +359,7 @@
 
     def __init__(self, working_tree, this_tree, base_tree, other_tree, 
                  interesting_ids=None, reprocess=False, show_base=False,
-                 pb=DummyProgress(), pp=None):
+                 pb=DummyProgress(), pp=None, change_reporter=None):
         """Initialize the merger object and perform the merge."""
         object.__init__(self)
         self.this_tree = working_tree
@@ -369,6 +371,7 @@
         self.show_base = show_base
         self.pb = pb
         self.pp = pp
+        self.change_reporter = change_reporter
         if self.pp is None:
             self.pp = ProgressPhase("Merge phase", 3, self.pb)
 
@@ -397,6 +400,9 @@
                 fs_conflicts = resolve_conflicts(self.tt, child_pb)
             finally:
                 child_pb.finished()
+            if change_reporter is not None:
+                from bzrlib import delta
+                delta.report_changes(self.tt._iter_changes(), change_reporter)
             self.cook_conflicts(fs_conflicts)
             for conflict in self.cooked_conflicts:
                 warning(conflict)
@@ -820,13 +826,14 @@
 
     def __init__(self, working_tree, this_tree, base_tree, other_tree, 
                  interesting_ids=None, pb=DummyProgress(), pp=None,
-                 reprocess=False):
+                 reprocess=False, change_reporter=None):
         self.this_revision_tree = self._get_revision_tree(this_tree)
         self.other_revision_tree = self._get_revision_tree(other_tree)
         super(WeaveMerger, self).__init__(working_tree, this_tree, 
                                           base_tree, other_tree, 
                                           interesting_ids=interesting_ids, 
-                                          pb=pb, pp=pp, reprocess=reprocess)
+                                          pb=pb, pp=pp, reprocess=reprocess,
+                                          change_reporter=change_reporter)
 
     def _get_revision_tree(self, tree):
         """Return a revision tree related to this tree.
@@ -933,7 +940,8 @@
                 other_rev_id=None,
                 interesting_files=None,
                 this_tree=None,
-                pb=DummyProgress()):
+                pb=DummyProgress(),
+                change_reporter=None):
     """Primary interface for merging. 
 
         typical use is probably 
@@ -947,7 +955,7 @@
              stacklevel=2)
         this_tree = this_branch.bzrdir.open_workingtree()
     merger = Merger(this_branch, other_tree, base_tree, this_tree=this_tree, 
-                    pb=pb)
+                    pb=pb, change_reporter=change_reporter)
     merger.backup_files = backup_files
     merger.merge_type = merge_type
     merger.interesting_ids = interesting_ids

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2007-02-13 20:33:57 +0000
+++ bzrlib/osutils.py	2007-02-17 01:42:57 +0000
@@ -260,6 +260,36 @@
     return _win32_fixdrive(_nt_abspath(unicode(path)).replace('\\', '/'))
 
 
+def _win98_abspath(path):
+    """Return the absolute version of a path.
+    Windows 98 safe implementation (python reimplementation
+    of Win32 API function GetFullPathNameW)
+    """
+    # Corner cases:
+    #   C:\path     => C:/path
+    #   C:/path     => C:/path
+    #   \\HOST\path => //HOST/path
+    #   //HOST/path => //HOST/path
+    #   path        => C:/cwd/path
+    #   /path       => C:/path
+    path = unicode(path)
+    # check for absolute path
+    drive = _nt_splitdrive(path)[0]
+    if drive == '' and path[:2] not in('//','\\\\'):
+        cwd = os.getcwdu()
+        # we cannot simply os.path.join cwd and path
+        # because os.path.join('C:','/path') produce '/path'
+        # and this is incorrect
+        if path[:1] in ('/','\\'):
+            cwd = _nt_splitdrive(cwd)[0]
+            path = path[1:]
+        path = cwd + '\\' + path
+    return _win32_fixdrive(_nt_normpath(path).replace('\\', '/'))
+
+if win32utils.winver == 'Windows 98':
+    _win32_abspath = _win98_abspath
+
+
 def _win32_realpath(path):
     # Real _nt_realpath doesn't have a problem with a unicode cwd
     return _win32_fixdrive(_nt_realpath(unicode(path)).replace('\\', '/'))

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2007-02-13 20:33:57 +0000
+++ bzrlib/repofmt/knitrepo.py	2007-02-17 01:42:57 +0000
@@ -183,6 +183,13 @@
         vf = self._revision_store.get_revision_file(self.get_transaction())
         return vf
 
+    def _get_history_vf(self):
+        """Get a versionedfile whose history graph reflects all revisions.
+
+        For knit repositories, this is the revision knit.
+        """
+        return self._get_revision_vf()
+
     @needs_write_lock
     def reconcile(self, other=None, thorough=False):
         """Reconcile this repository."""

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2007-02-13 20:33:57 +0000
+++ bzrlib/repository.py	2007-02-17 01:42:57 +0000
@@ -679,6 +679,32 @@
             done.add(revision_id)
         return result
 
+    def _get_history_vf(self):
+        """Get a versionedfile whose history graph reflects all revisions.
+
+        For weave repositories, this is the inventory weave.
+        """
+        return self.get_inventory_weave()
+
+    def iter_reverse_revision_history(self, revision_id):
+        """Iterate backwards through revision ids in the lefthand history
+
+        :param revision_id: The revision id to start with.  All its lefthand
+            ancestors will be traversed.
+        """
+        revision_id = osutils.safe_revision_id(revision_id)
+        if revision_id in (None, _mod_revision.NULL_REVISION):
+            return
+        next_id = revision_id
+        versionedfile = self._get_history_vf()
+        while True:
+            yield next_id
+            parents = versionedfile.get_parents(next_id)
+            if len(parents) == 0:
+                return
+            else:
+                next_id = parents[0]
+
     @needs_read_lock
     def get_revision_inventory(self, revision_id):
         """Return inventory of a past revision."""

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2007-02-01 23:48:08 +0000
+++ bzrlib/tests/__init__.py	2007-02-15 17:54:17 +0000
@@ -1424,10 +1424,10 @@
                     t.mkdir('.')
                 except errors.FileExists:
                     pass
+            if format is None:
+                format = 'default'
             if isinstance(format, basestring):
                 format = bzrdir.format_registry.make_bzrdir(format)
-            elif format is None:
-                format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
             return format.initialize_on_transport(t)
         except errors.UninitializableFormat:
             raise TestSkipped("Format %s is not initializable." % format)
@@ -1871,7 +1871,20 @@
             raise
     for name, plugin in bzrlib.plugin.all_plugins().items():
         if getattr(plugin, 'test_suite', None) is not None:
-            suite.addTest(plugin.test_suite())
+            default_encoding = sys.getdefaultencoding()
+            try:
+                plugin_suite = plugin.test_suite()
+            except ImportError, e:
+                bzrlib.trace.warning(
+                    'Unable to test plugin "%s": %s', name, e)
+            else:
+                suite.addTest(plugin_suite)
+            if default_encoding != sys.getdefaultencoding():
+                bzrlib.trace.warning(
+                    'Plugin "%s" tried to reset default encoding to: %s', name,
+                    sys.getdefaultencoding())
+                reload(sys)
+                sys.setdefaultencoding(default_encoding)
     return suite
 
 

=== modified file 'bzrlib/tests/blackbox/test_bound_branches.py'
--- bzrlib/tests/blackbox/test_bound_branches.py	2007-02-06 02:33:42 +0000
+++ bzrlib/tests/blackbox/test_bound_branches.py	2007-02-14 14:57:43 +0000
@@ -102,6 +102,31 @@
 
         self.run_bzr('unbind', retcode=3)
 
+    def test_bind_branch6(self):
+        branch1 = self.make_branch('branch1', format='experimental-branch6')
+        os.chdir('branch1')
+        error = self.run_bzr('bind', retcode=3)[1]
+        self.assertContainsRe(error, 'no previous location known')
+
+    def setup_rebind(self, format):
+        branch1 = self.make_branch('branch1')
+        branch2 = self.make_branch('branch2', format=format)
+        branch2.bind(branch1)
+        branch2.unbind()
+
+    def test_rebind_branch6(self):
+        self.setup_rebind('experimental-branch6')
+        os.chdir('branch2')
+        self.run_bzr('bind')
+        b = Branch.open('.')
+        self.assertContainsRe(b.get_bound_location(), '\/branch1\/$')
+
+    def test_rebind_branch5(self):
+        self.setup_rebind('knit')
+        os.chdir('branch2')
+        error = self.run_bzr('bind', retcode=3)[1]
+        self.assertContainsRe(error, 'old locations')
+
     def init_meta_branch(self, path):
         format = bzrdir.format_registry.make_bzrdir('knit')
         return BzrDir.create_branch_convenience(path, format=format)

=== modified file 'bzrlib/tests/blackbox/test_info.py'
--- bzrlib/tests/blackbox/test_info.py	2007-02-06 06:27:24 +0000
+++ bzrlib/tests/blackbox/test_info.py	2007-02-15 14:08:23 +0000
@@ -150,7 +150,7 @@
 Format:
        control: Meta directory format 1
   working tree: %s
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -176,6 +176,7 @@
        branch1.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
        bound_tree._format.get_format_description(),      
+       branch3._format.get_format_description(),
        branch3.repository._format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
@@ -469,7 +470,7 @@
 
 Format:
        control: Meta directory format 1
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 Branch history:
@@ -479,6 +480,7 @@
          0 revisions
          0 KiB
 """ % (branch.bzrdir.root_transport.base,
+       format.get_branch_format().get_format_description(),
        format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
@@ -519,7 +521,7 @@
 
 Format:
        control: Meta directory format 1
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 Branch history:
@@ -529,6 +531,7 @@
          0 revisions
          0 KiB
 """ % (repo.bzrdir.root_transport.base,
+       format.get_branch_format().get_format_description(),
        format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
@@ -563,7 +566,7 @@
 Format:
        control: Meta directory format 1
   working tree: Working tree format 3
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -588,6 +591,7 @@
          %d KiB
 """ % (tree2.bzrdir.root_transport.base,
        repo.bzrdir.root_transport.base,
+       format.get_branch_format().get_format_description(),
        format.repository_format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
@@ -606,7 +610,7 @@
 Format:
        control: Meta directory format 1
   working tree: Working tree format 3
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 Branch is out of date: missing 1 revision.
@@ -629,6 +633,7 @@
          0 KiB
 """ % (tree3.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
+       format.get_branch_format().get_format_description(),
        format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
@@ -646,7 +651,7 @@
 Format:
        control: Meta directory format 1
   working tree: Working tree format 3
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -670,6 +675,7 @@
          1 revision
          %d KiB
 """ % (tree3.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
+       format.get_branch_format().get_format_description(),
        format.repository_format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
@@ -692,7 +698,7 @@
 Format:
        control: Meta directory format 1
   working tree: Working tree format 3
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 Working tree is out of date: missing 1 revision.
@@ -719,6 +725,7 @@
          %d KiB
 """ % (tree2.bzrdir.root_transport.base,
        repo.bzrdir.root_transport.base,
+       format.get_branch_format().get_format_description(),
        format.repository_format.get_format_description(),
        datestring_first, datestring_last,
        # poking at _revision_store isn't all that clean, but neither is
@@ -736,7 +743,7 @@
 
 Format:
        control: Meta directory format 1
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 Branch history:
@@ -750,6 +757,7 @@
          2 revisions
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
+       format.get_branch_format().get_format_description(),
        format.repository_format.get_format_description(),
        datestring_first, datestring_last,
        # poking at _revision_store isn't all that clean, but neither is
@@ -821,7 +829,7 @@
 Format:
        control: Meta directory format 1
   working tree: Working tree format 3
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -842,6 +850,7 @@
          0 revisions
          0 KiB
 """ % (repo.bzrdir.root_transport.base,
+       format.get_branch_format().get_format_description(),
        format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
@@ -862,7 +871,7 @@
 Format:
        control: Meta directory format 1
   working tree: Working tree format 3
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -885,6 +894,7 @@
          1 revision
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
+       format.get_branch_format().get_format_description(),
        format.repository_format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
@@ -906,7 +916,7 @@
 Format:
        control: Meta directory format 1
   working tree: Working tree format 3
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -928,6 +938,7 @@
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
+       format.get_branch_format().get_format_description(),
        format.repository_format.get_format_description(),
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
@@ -950,7 +961,7 @@
 Format:
        control: Meta directory format 1
   working tree: Working tree format 3
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -974,6 +985,7 @@
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
+       format.get_branch_format().get_format_description(),
        format.repository_format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
@@ -1045,7 +1057,7 @@
 Format:
        control: Meta directory format 1
   working tree: Working tree format 3
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -1065,6 +1077,7 @@
          0 revisions
          0 KiB
 """ % (repo.bzrdir.root_transport.base,
+       format.get_branch_format().get_format_description(),
        format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
@@ -1144,7 +1157,7 @@
 Format:
        control: Meta directory format 1
   working tree: %s
-        branch: Branch format 5
+        branch: %s
     repository: %s
 %s
 In the working tree:
@@ -1166,6 +1179,7 @@
 """ %  (tree_data,
         branch_data,
         lco_tree._format.get_format_description(),
+        lco_tree.branch._format.get_format_description(),
         lco_tree.branch.repository._format.get_format_description(),
         expected_lock_output,
         verbose_info,

=== modified file 'bzrlib/tests/blackbox/test_init.py'
--- bzrlib/tests/blackbox/test_init.py	2007-02-05 11:13:22 +0000
+++ bzrlib/tests/blackbox/test_init.py	2007-02-13 19:22:47 +0000
@@ -20,6 +20,9 @@
 import os
 import re
 
+from bzrlib import (
+    branch as _mod_branch,
+    )
 from bzrlib.bzrdir import BzrDirMetaFormat1
 from bzrlib.tests.blackbox import ExternalBase
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
@@ -133,3 +136,14 @@
 
         # rely on SFTPServer get_url() pointing at '.'
         self.run_bzr_error(['Already a branch'], 'init', self.get_url())
+
+    def test_init_append_revisions_only(self):
+        self.run_bzr('init', '--experimental-branch6', 'normal_branch6')
+        branch = _mod_branch.Branch.open('normal_branch6')
+        self.assertEqual(False, branch._get_append_revisions_only())
+        self.run_bzr('init', '--append-revisions-only',
+                     '--experimental-branch6', 'branch6')
+        branch = _mod_branch.Branch.open('branch6')
+        self.assertEqual(True, branch._get_append_revisions_only())
+        self.run_bzr_error(['cannot be set to append-revisions-only'], 'init',
+            '--append-revisions-only', '--knit', 'knit')

=== modified file 'bzrlib/tests/blackbox/test_merge.py'
--- bzrlib/tests/blackbox/test_merge.py	2007-02-06 15:32:30 +0000
+++ bzrlib/tests/blackbox/test_merge.py	2007-02-14 21:57:40 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -169,7 +169,7 @@
         
         base = urlutils.local_path_from_url(branch_a.base)
         self.assertEquals(out, 'Merging from remembered location %s\n' % (base,))
-        self.assertEquals(err, 'All changes applied successfully.\n')
+        self.assertEquals(err, '+N  b\nAll changes applied successfully.\n')
         self.assertEquals(abspath(branch_b.get_parent()), abspath(parent))
         # re-open tree as external runbzr modified it
         tree_b = branch_b.bzrdir.open_workingtree()
@@ -177,7 +177,7 @@
         # test explicit --remember
         out, err = self.runbzr('merge ../branch_c --remember')
         self.assertEquals(out, '')
-        self.assertEquals(err, 'All changes applied successfully.\n')
+        self.assertEquals(err, '+N  c\nAll changes applied successfully.\n')
         self.assertEquals(abspath(branch_b.get_parent()),
                           abspath(branch_c.bzrdir.root_transport.base))
         # re-open tree as external runbzr modified it
@@ -232,13 +232,14 @@
         tree_a.rename_one('file_1', 'file_i')
         tree_a.commit('commit 2')
         tree_a.rename_one('file_2', 'file_ii')
-        os.chdir('b')
-        self.run_bzr('merge', '../a', '--uncommitted')
-        self.failUnlessExists('file_1')
-        self.failUnlessExists('file_ii')
+        ## os.chdir('b')
+        self.run_bzr('merge', 'a', '--uncommitted', '-d', 'b')
+        self.failUnlessExists('b/file_1')
+        self.failUnlessExists('b/file_ii')
         tree_b.revert([])
-        self.run_bzr_error(('Cannot use --uncommitted and --revision',), 
-                           'merge', '../a', '--uncommitted', '-r1')
+        self.run_bzr_error(('Cannot use --uncommitted and --revision',),
+                           'merge', '/a', '--uncommitted', '-r1',
+                           '-d', 'b')
 
     def pullable_branch(self):
         os.mkdir('a')

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2007-02-14 21:57:40 +0000
@@ -1,5 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
-# -*- coding: utf-8 -*-
+# Copyright (C) 2005, 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +23,7 @@
 from bzrlib.branch import Branch
 from bzrlib.tests.blackbox import ExternalBase
 from bzrlib.uncommit import uncommit
+from bzrlib import urlutils
 
 
 class TestPull(ExternalBase):
@@ -96,6 +96,19 @@
         self.runbzr('pull ../b')
         self.runbzr('pull ../b')
 
+    def test_pull_dash_d(self):
+        os.mkdir('a')
+        os.chdir('a')
+        self.example_branch()
+        self.runbzr('init ../b')
+        self.runbzr('init ../c')
+        # pull into that branch
+        self.runbzr('pull -d ../b .')
+        # pull into a branch specified by a url
+        c_url = urlutils.local_path_to_url('../c')
+        self.assertStartsWith(c_url, 'file://')
+        self.runbzr('pull -d %s .' % c_url)
+
     def test_pull_revision(self):
         """Pull some changes from one branch to another."""
         os.mkdir('a')
@@ -286,7 +299,7 @@
         os.chdir('../branch_b')
         output = self.run_bzr('pull', '../bundle')
         self.assertEqual('', output[0])
-        self.assertEqual('All changes applied successfully.\n'
+        self.assertEqual(' M  a\nAll changes applied successfully.\n'
                          '1 revision(s) pulled.\n', output[1])
 
         self.assertEqualDiff(tree_a.branch.revision_history(),

=== modified file 'bzrlib/tests/blackbox/test_push.py'
--- bzrlib/tests/blackbox/test_push.py	2007-02-13 11:48:14 +0000
+++ bzrlib/tests/blackbox/test_push.py	2007-02-14 09:51:20 +0000
@@ -1,5 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
-# -*- coding: utf-8 -*-
+# Copyright (C) 2005, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,8 +21,8 @@
 
 from bzrlib import (
     errors,
+    urlutils,
     )
-import bzrlib
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDirMetaFormat1
 from bzrlib.osutils import abspath
@@ -100,7 +99,7 @@
         out, err = self.run_bzr('push', 'pushed-location')
         self.assertEqual('', out)
         self.assertEqual('0 revision(s) pushed.\n', err)
-        b2 = bzrlib.branch.Branch.open('pushed-location')
+        b2 = Branch.open('pushed-location')
         self.assertEndsWith(b2.base, 'pushed-location/')
 
     def test_push_new_branch_revision_count(self):
@@ -164,6 +163,16 @@
         t.commit('commit filename')
         self.run_bzr('push', '../new-tree')
 
+    def test_push_dash_d(self):
+        t = self.make_branch_and_tree('from')
+        t.commit(allow_pointless=True,
+                message='first commit')
+        self.runbzr('push -d from to-one')
+        self.failUnlessExists('to-one')
+        self.runbzr('push -d %s %s' 
+            % tuple(map(urlutils.local_path_to_url, ['from', 'to-two'])))
+        self.failUnlessExists('to-two')
+
     def create_simple_tree(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/a'])

=== modified file 'bzrlib/tests/blackbox/test_revert.py'
--- bzrlib/tests/blackbox/test_revert.py	2007-02-01 20:36:13 +0000
+++ bzrlib/tests/blackbox/test_revert.py	2007-02-09 17:09:47 +0000
@@ -106,10 +106,10 @@
     def test_revert_chatter(self):
         self._prepare_rename_mod_tree()
         chatter = self.run_bzr('revert')[1]
-        self.assertEqualDiff('R   j/ => f/\n'
-                             'R   j/b => a/b\n'
+        self.assertEqualDiff('R   j/b => a/b\n'
                              'R   j/d/ => a/d/\n'
                              'R   j/e => a/d/e\n'
+                             'R   j/ => f/\n'
                              'R   a/g => f/g\n'
                              'R   h => f/h\n', chatter)
     def test_revert(self):

=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2007-02-13 20:33:57 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2007-02-17 01:42:57 +0000
@@ -82,7 +82,13 @@
 
     def test_append_revisions(self):
         """Test appending more than one revision"""
+        wt = self.make_branch_and_tree('tree')
+        wt.commit('f', rev_id='rev1')
+        wt.commit('f', rev_id='rev2')
+        wt.commit('f', rev_id='rev3')
+
         br = self.get_branch()
+        br.fetch(wt.branch)
         br.append_revision("rev1")
         self.assertEquals(br.revision_history(), ["rev1",])
         br.append_revision("rev2", "rev3")
@@ -90,7 +96,13 @@
         self.assertRaises(errors.ReservedId, br.append_revision, 'current:')
 
     def test_revision_ids_are_utf8(self):
+        wt = self.make_branch_and_tree('tree')
+        wt.commit('f', rev_id='rev1')
+        wt.commit('f', rev_id='rev2')
+        wt.commit('f', rev_id='rev3')
+
         br = self.get_branch()
+        br.fetch(wt.branch)
         br.set_revision_history(['rev1', 'rev2', 'rev3'])
         rh = br.revision_history()
         self.assertEqual(['rev1', 'rev2', 'rev3'], rh)
@@ -224,6 +236,19 @@
         branch_d = branch_b.clone(repo_d.bzrdir)
         self.assertEqual(random_parent, branch_d.get_parent())
 
+    def test_copy_content_incomplete(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.root_transport)
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        source = self.make_branch_and_tree('source')
+        # this gives us an incomplete repository
+        tree.bzrdir.open_repository().copy_content_into(
+            source.branch.repository)
+        tree.commit('revision 2', rev_id='2', allow_pointless=True)
+        tree.bzrdir.open_branch().copy_content_into(source.branch)
+
+
     def test_sprout_branch_nickname(self):
         # test the nick name is reset always
         raise TestSkipped('XXX branch sprouting is not yet tested..')
@@ -588,20 +613,9 @@
         self.assertEqual("foo", self.get_branch().get_push_location())
 
     def test_set_push_location(self):
-        from bzrlib.config import (locations_config_filename,
-                                   ensure_config_dir_exists)
-        ensure_config_dir_exists()
-        fn = locations_config_filename()
         branch = self.get_branch()
         branch.set_push_location('foo')
-        local_path = urlutils.local_path_from_url(branch.base[:-1])
-        self.assertFileEqual("[%s]\n"
-                             "push_location = foo\n"
-                             "push_location:policy = norecurse" % local_path,
-                             fn)
-
-    # TODO RBC 20051029 test getting a push location from a branch in a 
-    # recursive section - that is, it appends the branch name.
+        self.assertEqual('foo', branch.get_push_location())
 
 
 class TestFormat(TestCaseWithBranch):
@@ -642,3 +656,52 @@
                          branch.BranchFormat.find_format(opened_control))
 
 
+class TestBound(TestCaseWithBranch):
+
+    def test_bind_unbind(self):
+        branch = self.make_branch('1')
+        branch2 = self.make_branch('2')
+        try:
+            branch.bind(branch2)
+        except errors.UpgradeRequired:
+            raise TestSkipped('Format does not support binding')
+        self.assertTrue(branch.unbind())
+        self.assertFalse(branch.unbind())
+        self.assertIs(None, branch.get_bound_location())
+
+    def test_old_bound_location(self):
+        branch = self.make_branch('branch1')
+        try:
+            self.assertIs(None, branch.get_old_bound_location())
+        except errors.UpgradeRequired:
+            raise TestSkipped('Format does not store old bound locations')
+        branch2 = self.make_branch('branch2')
+        branch.bind(branch2)
+        self.assertIs(None, branch.get_old_bound_location())
+        branch.unbind()
+        self.assertContainsRe(branch.get_old_bound_location(), '\/branch2\/$')
+
+
+class TestStrict(TestCaseWithBranch):
+
+    def test_strict_history(self):
+        tree1 = self.make_branch_and_tree('tree1')
+        try:
+            tree1.branch.set_append_revisions_only(True)
+        except errors.UpgradeRequired:
+            raise TestSkipped('Format does not support strict history')
+        tree1.commit('empty commit')
+        tree2 = tree1.bzrdir.sprout('tree2').open_workingtree()
+        tree2.commit('empty commit 2')
+        tree1.pull(tree2.branch)
+        tree1.commit('empty commit 3')
+        tree2.commit('empty commit 4')
+        self.assertRaises(errors.DivergedBranches, tree1.pull, tree2.branch)
+        tree2.merge_from_branch(tree1.branch)
+        tree2.commit('empty commit 5')
+        self.assertRaises(errors.AppendRevisionsOnlyViolation, tree1.pull,
+                          tree2.branch)
+        tree3 = tree1.bzrdir.sprout('tree3').open_workingtree()
+        tree3.merge_from_branch(tree2.branch)
+        tree3.commit('empty commit 6')
+        tree2.pull(tree3.branch)

=== modified file 'bzrlib/tests/branch_implementations/test_hooks.py'
--- bzrlib/tests/branch_implementations/test_hooks.py	2007-02-06 02:33:42 +0000
+++ bzrlib/tests/branch_implementations/test_hooks.py	2007-02-11 20:40:48 +0000
@@ -42,7 +42,12 @@
             [('set_rh', branch, [], True)])
 
     def test_set_rh_nonempty_history(self):
-        branch = self.make_branch('source')
+        tree = self.make_branch_and_memory_tree('source')
+        tree.lock_write()
+        tree.add('')
+        tree.commit('empty commit', rev_id='foo')
+        tree.unlock()
+        branch = tree.branch
         Branch.hooks.install_hook('set_rh', self.capture_set_rh_hook)
         branch.set_revision_history([u'foo'])
         self.assertEqual(self.hook_calls,

=== modified file 'bzrlib/tests/branch_implementations/test_parent.py'
--- bzrlib/tests/branch_implementations/test_parent.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/branch_implementations/test_parent.py	2007-01-16 14:26:41 +0000
@@ -42,7 +42,7 @@
         url = 'http://bazaar-vcs.org/bzr/bzr.dev'
         b.set_parent(url)
         self.assertEqual(url, b.get_parent())
-        self.assertEqual(url, b.control_files.get('parent').read().strip('\n'))
+        self.assertEqual(url, b._get_parent_location())
 
         b.set_parent(None)
         self.assertEqual(None, b.get_parent())
@@ -52,15 +52,13 @@
         self.assertEqual(local_path_to_url('../other_branch'), b.get_parent())
         path = local_path_to_url('../yanb')
         b.set_parent(path)
-        self.assertEqual('../yanb',
-            b.control_files.get('parent').read().strip('\n'))
+        self.assertEqual('../yanb', b._get_parent_location())
         self.assertEqual(path, b.get_parent())
 
 
         self.assertRaises(bzrlib.errors.InvalidURL, b.set_parent, u'\xb5')
         b.set_parent(escape(u'\xb5'))
-        self.assertEqual('%C2%B5',
-            b.control_files.get('parent').read().strip('\n'))
+        self.assertEqual('%C2%B5', b._get_parent_location())
 
         self.assertEqual(b.base + '%C2%B5', b.get_parent())
 
@@ -70,7 +68,7 @@
             #       paths as well? Nobody has complained about it.
             pass
         else:
-            b.control_files.put('parent', cStringIO.StringIO('/local/abs/path'))
+            b._set_parent_location('/local/abs/path')
             self.assertEqual('file:///local/abs/path', b.get_parent())
 
     def test_get_invalid_parent(self):
@@ -82,7 +80,7 @@
         # Force the relative path to be something invalid
         # This should attempt to go outside the filesystem
         path = ('../'*(n_dirs+5)) + 'foo'
-        b.control_files.put('parent', cStringIO.StringIO(path))
+        b._set_parent_location(path)
 
         # With an invalid branch parent, just return None
         self.assertRaises(bzrlib.errors.InaccessibleParent, b.get_parent)

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2007-02-06 06:27:24 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2007-02-13 18:58:39 +0000
@@ -97,11 +97,14 @@
         directories = ['.']
         while directories:
             dir = directories.pop()
-            for path in source.list_dir(dir):
+            for path in set(source.list_dir(dir) + target.list_dir(dir)):
                 path = dir + '/' + path
                 if path in ignore_list:
                     continue
-                stat = source.stat(path)
+                try:
+                    stat = source.stat(path)
+                except errors.NoSuchFile:
+                    self.fail('%s not in source' % path)
                 if S_ISDIR(stat.st_mode):
                     self.assertTrue(S_ISDIR(target.stat(path).st_mode))
                     directories.append(path)
@@ -177,7 +180,8 @@
         dir = self.make_bzrdir('source')
         target = dir.clone(self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/merge-hashes'])
     
     def test_clone_bzrdir_empty_force_new_ignored(self):
         # the force_new_repo parameter should have no effect on an empty
@@ -185,7 +189,8 @@
         dir = self.make_bzrdir('source')
         target = dir.clone(self.get_url('target'), force_new_repo=True)
         self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/merge-hashes'])
     
     def test_clone_bzrdir_repository(self):
         tree = self.make_branch_and_tree('commit_tree')
@@ -199,7 +204,9 @@
         target = dir.clone(self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/repository/inventory.knit',
+                                    [
+                                     './.bzr/merge-hashes',
+                                     './.bzr/repository/inventory.knit',
                                      ])
 
 
@@ -315,10 +322,13 @@
         target = dir.clone(self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/stat-cache',
+                                    [
+                                     './.bzr/basis-inventory-cache',
                                      './.bzr/checkout/stat-cache',
+                                     './.bzr/merge-hashes',
                                      './.bzr/repository/inventory.knit',
-                                     ])
+                                     './.bzr/stat-cache',
+                                    ])
 
     def test_clone_bzrdir_branch_and_repo_into_shared_repo(self):
         # by default cloning into a shared repo uses the shared repo.
@@ -406,6 +416,8 @@
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     ['./.bzr/stat-cache',
                                      './.bzr/checkout/stat-cache',
+                                     './.bzr/checkout/merge-hashes',
+                                     './.bzr/merge-hashes',
                                      './.bzr/repository/inventory.knit',
                                      ])
 
@@ -422,6 +434,8 @@
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     ['./.bzr/stat-cache',
                                      './.bzr/checkout/stat-cache',
+                                     './.bzr/checkout/merge-hashes',
+                                     './.bzr/merge-hashes',
                                      './.bzr/repository/inventory.knit',
                                      ])
 
@@ -429,6 +443,8 @@
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     ['./.bzr/stat-cache',
                                      './.bzr/checkout/stat-cache',
+                                     './.bzr/checkout/merge-hashes',
+                                     './.bzr/merge-hashes',
                                      './.bzr/repository/inventory.knit',
                                      ])
 
@@ -451,6 +467,8 @@
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     ['./.bzr/stat-cache',
                                      './.bzr/checkout/stat-cache',
+                                     './.bzr/checkout/merge-hashes',
+                                     './.bzr/merge-hashes',
                                      './.bzr/repository/inventory.knit',
                                      ])
 
@@ -546,8 +564,12 @@
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # testing inventory isn't reasonable for repositories
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/repository/inventory.knit',
-                                     './.bzr/inventory'
+                                    [
+                                     './.bzr/branch',
+                                     './.bzr/checkout',
+                                     './.bzr/inventory',
+                                     './.bzr/parent',
+                                     './.bzr/repository/inventory.knit',
                                      ])
         try:
             # If we happen to have a tree, we'll guarantee everything
@@ -676,11 +698,18 @@
         target = self.sproutOrSkip(dir, self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/stat-cache',
+                                    [
+                                     './.bzr/basis-inventory-cache',
+                                     './.bzr/branch/branch.conf',
+                                     './.bzr/branch/parent',
+                                     './.bzr/checkout',
+                                     './.bzr/checkout/inventory',
                                      './.bzr/checkout/stat-cache',
                                      './.bzr/inventory',
-                                     './.bzr/checkout/inventory',
+                                     './.bzr/parent',
                                      './.bzr/repository/inventory.knit',
+                                     './.bzr/stat-cache',
+                                     './foo',
                                      ])
 
     def test_sprout_bzrdir_branch_and_repo_shared(self):
@@ -818,11 +847,15 @@
         target = self.sproutOrSkip(dir, self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/stat-cache',
+                                    [
+                                     './.bzr/branch/branch.conf',
+                                     './.bzr/branch/parent',
                                      './.bzr/checkout/stat-cache',
+                                     './.bzr/checkout/inventory',
                                      './.bzr/inventory',
-                                     './.bzr/checkout/inventory',
+                                     './.bzr/parent',
                                      './.bzr/repository/inventory.knit',
+                                     './.bzr/stat-cache',
                                      ])
 
     def test_sprout_bzrdir_tree_branch_reference(self):

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2007-02-06 02:33:42 +0000
+++ bzrlib/tests/test_branch.py	2007-02-15 14:08:23 +0000
@@ -24,10 +24,15 @@
 
 from StringIO import StringIO
 
+from bzrlib import (
+    branch as _mod_branch,
+    bzrdir,
+    errors,
+    urlutils,
+    )
 import bzrlib.branch
 from bzrlib.branch import (BzrBranch5, 
                            BzrBranchFormat5)
-import bzrlib.bzrdir as bzrdir
 from bzrlib.bzrdir import (BzrDirMetaFormat1, BzrDirMeta1, 
                            BzrDir, BzrDirFormat)
 from bzrlib.errors import (NotBranchError,
@@ -77,6 +82,22 @@
         finally:
             branch.unlock()
 
+    def test_set_push_location(self):
+        from bzrlib.config import (locations_config_filename,
+                                   ensure_config_dir_exists)
+        ensure_config_dir_exists()
+        fn = locations_config_filename()
+        branch = self.make_branch('.', format='knit')
+        branch.set_push_location('foo')
+        local_path = urlutils.local_path_from_url(branch.base[:-1])
+        self.assertFileEqual("[%s]\n"
+                             "push_location = foo\n"
+                             "push_location:policy = norecurse" % local_path,
+                             fn)
+
+    # TODO RBC 20051029 test getting a push location from a branch in a
+    # recursive section - that is, it appends the branch name.
+
 
 class SampleBranchFormat(bzrlib.branch.BranchFormat):
     """A sample format
@@ -141,10 +162,93 @@
         bzrlib.branch.BranchFormat.register_format(format)
         # which branch.Open will refuse (not supported)
         self.assertRaises(UnsupportedFormatError, bzrlib.branch.Branch.open, self.get_url())
+        self.make_branch_and_tree('foo')
         # but open_downlevel will work
         self.assertEqual(format.open(dir), bzrdir.BzrDir.open(self.get_url()).open_branch(unsupported=True))
         # unregister the format
         bzrlib.branch.BranchFormat.unregister_format(format)
+        self.make_branch_and_tree('bar')
+
+    def test_checkout_format(self):
+        branch = self.make_repository('repository', shared=True)
+        branch = self.make_branch('repository/branch',
+            format='metaweave')
+        tree = branch.create_checkout('checkout')
+        self.assertIs(tree.branch.__class__, _mod_branch.BzrBranch5)
+
+
+class TestBranch6(TestCaseWithTransport):
+
+    def test_creation(self):
+        format = BzrDirMetaFormat1()
+        format.set_branch_format(_mod_branch.BzrBranchFormat6())
+        branch = self.make_branch('a', format=format)
+        self.assertIsInstance(branch, _mod_branch.BzrBranch6)
+        branch = self.make_branch('b', format='experimental-branch6')
+        self.assertIsInstance(branch, _mod_branch.BzrBranch6)
+        branch = _mod_branch.Branch.open('a')
+        self.assertIsInstance(branch, _mod_branch.BzrBranch6)
+
+    def test_layout(self):
+        branch = self.make_branch('a', format='experimental-branch6')
+        self.failUnlessExists('a/.bzr/branch/last-revision')
+        self.failIfExists('a/.bzr/branch/revision-history')
+
+    def test_config(self):
+        """Ensure that all configuration data is stored in the branch"""
+        branch = self.make_branch('a', format='experimental-branch6')
+        branch.set_parent('http://bazaar-vcs.org')
+        self.failIfExists('a/.bzr/branch/parent')
+        self.assertEqual('http://bazaar-vcs.org', branch.get_parent())
+        branch.set_push_location('sftp://bazaar-vcs.org')
+        config = branch.get_config()._get_branch_data_config()
+        self.assertEqual('sftp://bazaar-vcs.org',
+                         config.get_user_option('push_location'))
+        branch.set_bound_location('ftp://bazaar-vcs.org')
+        self.failIfExists('a/.bzr/branch/bound')
+        self.assertEqual('ftp://bazaar-vcs.org', branch.get_bound_location())
+
+    def test_set_revision_history(self):
+        tree = self.make_branch_and_memory_tree('.',
+            format='experimental-branch6')
+        tree.lock_write()
+        try:
+            tree.add('.')
+            tree.commit('foo', rev_id='foo')
+            tree.commit('bar', rev_id='bar')
+            tree.branch.set_revision_history(['foo', 'bar'])
+            tree.branch.set_revision_history(['foo'])
+            self.assertRaises(errors.NotLefthandHistory,
+                              tree.branch.set_revision_history, ['bar'])
+        finally:
+            tree.unlock()
+
+    def test_append_revision(self):
+        tree = self.make_branch_and_tree('branch1',
+            format='experimental-branch6')
+        tree.lock_write()
+        try:
+            tree.add('.')
+            tree.commit('foo', rev_id='foo')
+            tree.commit('bar', rev_id='bar')
+            tree.commit('baz', rev_id='baz')
+            tree.set_last_revision('bar')
+            tree.branch.set_last_revision_info(2, 'bar')
+            tree.commit('qux', rev_id='qux')
+            tree.add_parent_tree_id('baz')
+            tree.commit('qux', rev_id='quxx')
+            tree.branch.set_last_revision_info(0, 'null:')
+            self.assertRaises(errors.NotLeftParentDescendant,
+                              tree.branch.append_revision, 'bar')
+            tree.branch.append_revision('foo')
+            self.assertRaises(errors.NotLeftParentDescendant,
+                              tree.branch.append_revision, 'baz')
+            tree.branch.append_revision('bar')
+            tree.branch.append_revision('baz')
+            self.assertRaises(errors.NotLeftParentDescendant,
+                              tree.branch.append_revision, 'quxx')
+        finally:
+            tree.unlock()
 
 
 class TestBranchReference(TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2007-02-07 09:11:31 +0000
+++ bzrlib/tests/test_bzrdir.py	2007-02-15 14:08:23 +0000
@@ -71,14 +71,21 @@
             ' repositories', deprecated=True)
         my_format_registry.register_lazy('lazy', 'bzrlib.bzrdir', 
             'BzrDirFormat6', 'Format registered lazily', deprecated=True)
-        my_format_registry.register_metadir('knit', 'RepositoryFormatKnit1',
+        my_format_registry.register_metadir('knit',
+            'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
             'Format using knits',
-            repo_module='bzrlib.repofmt.knitrepo')
+            )
         my_format_registry.set_default('knit')
-        my_format_registry.register_metadir('experimental-knit2',
-                                            'RepositoryFormatKnit2',
-            'Experimental successor to knit.  Use at your own risk.',
-            repo_module='bzrlib.repofmt.knitrepo')
+        my_format_registry.register_metadir(
+            'experimental-knit2',
+            'bzrlib.repofmt.knitrepo.RepositoryFormatKnit2',
+            'Experimental successor to knit.  Use at your own risk.',
+            )
+        my_format_registry.register_metadir(
+            'branch6',
+            'bzrlib.repofmt.knitrepo.RepositoryFormatKnit2',
+            'Experimental successor to knit.  Use at your own risk.',
+            branch_format='BzrBranchFormat6')
         return my_format_registry
 
     def test_format_registry(self):
@@ -93,6 +100,9 @@
         my_bzrdir = my_format_registry.make_bzrdir('knit')
         self.assertIsInstance(my_bzrdir.repository_format, 
             knitrepo.RepositoryFormatKnit1)
+        my_bzrdir = my_format_registry.make_bzrdir('branch6')
+        self.assertIsInstance(my_bzrdir.get_branch_format(),
+                              bzrlib.branch.BzrBranchFormat6)
 
     def test_get_help(self):
         my_format_registry = self.make_format_registry()

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2006-12-07 16:21:39 +0000
+++ bzrlib/tests/test_config.py	2007-01-16 13:12:54 +0000
@@ -413,7 +413,7 @@
 
     def test_config_creates_local(self):
         """Creating a new entry in config uses a local path."""
-        branch = self.make_branch('branch')
+        branch = self.make_branch('branch', format='knit')
         branch.set_push_location('http://foobar')
         locations = config.locations_config_filename()
         local_path = osutils.getcwd().encode('utf8')

=== modified file 'bzrlib/tests/test_delta.py'
--- bzrlib/tests/test_delta.py	2007-02-06 17:12:12 +0000
+++ bzrlib/tests/test_delta.py	2007-02-08 22:42:45 +0000
@@ -144,6 +144,7 @@
         self.assertChangesEqual(exe_change=False, modified='kind changed',
                                 executable=(False, True),
                                 kind=('directory', 'file'))
+        self.assertChangesEqual(parent_id=('pid', None))
 
         # Now make sure they all work together
         self.assertChangesEqual(versioned_change='removed',

=== modified file 'bzrlib/tests/test_fetch.py'
--- bzrlib/tests/test_fetch.py	2007-02-07 09:11:31 +0000
+++ bzrlib/tests/test_fetch.py	2007-02-13 18:58:39 +0000
@@ -96,10 +96,12 @@
     br_a2.append_revision('a-b-c')
     self.assertRaises(bzrlib.errors.InstallFailed, br_a3.fetch, br_a2)
 
-    # TODO: jam 20051218 Branch should no longer allow append_revision for revisions
-    #       which don't exist. So this test needs to be rewritten
-    #       RBC 20060403 the way to do this is to uncommit the revision from the
-    #           repository after the commit
+    # TODO: ADHB 20070116 Perhaps set_last_revision shouldn't accept
+    #       revisions which are not present?  In that case, this test
+    #       must be rewritten.
+    #
+    #       RBC 20060403 the way to do this is to uncommit the revision from
+    #       the repository after the commit
 
     #TODO: test that fetch correctly does reweaving when needed. RBC 20051008
     # Note that this means - updating the weave when ghosts are filled in to 
@@ -272,7 +274,10 @@
         self.assertEqual(1, self._count_log_matches('inventory.kndx', http_logs))
         # this r-h check test will prevent regressions, but it currently already 
         # passes, before the patch to cache-rh is applied :[
-        self.assertEqual(1, self._count_log_matches('revision-history', http_logs))
+        self.assertTrue(1 >= self._count_log_matches('revision-history',
+                                                     http_logs))
+        self.assertTrue(1 >= self._count_log_matches('last-revision',
+                                                     http_logs))
         # FIXME naughty poking in there.
         self.get_readonly_server().logs = []
         # check there is nothing more to fetch
@@ -285,5 +290,8 @@
         self.assertEqual(1, self._count_log_matches('branch-format', http_logs))
         self.assertEqual(1, self._count_log_matches('branch/format', http_logs))
         self.assertEqual(1, self._count_log_matches('repository/format', http_logs))
-        self.assertEqual(1, self._count_log_matches('revision-history', http_logs))
+        self.assertTrue(1 >= self._count_log_matches('revision-history',
+                                                     http_logs))
+        self.assertTrue(1 >= self._count_log_matches('last-revision',
+                                                     http_logs))
         self.assertEqual(4, len(http_logs))

=== modified file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	2007-02-13 05:22:39 +0000
+++ bzrlib/tests/test_options.py	2007-02-14 07:10:27 +0000
@@ -168,8 +168,10 @@
     def test_help(self):
         registry = bzrdir.BzrDirFormatRegistry()
         registry.register_metadir('one', 'RepositoryFormat7', 'one help')
-        registry.register_metadir('two', 'RepositoryFormatKnit1', 'two help',
-            repo_module='bzrlib.repofmt.knitrepo')
+        registry.register_metadir('two',
+            'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
+            'two help',
+            )
         registry.set_default('one')
         options = [option.RegistryOption('format', 'format help', registry,
                    str, value_switches=True, title='Formats')]
@@ -191,8 +193,10 @@
                          [('hello', None, 'GAR', 'fg')])
         registry = bzrdir.BzrDirFormatRegistry()
         registry.register_metadir('one', 'RepositoryFormat7', 'one help')
-        registry.register_metadir('two', 'RepositoryFormatKnit1', 'two help',
-            repo_module='bzrlib.repofmt.knitrepo')
+        registry.register_metadir('two',
+                'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
+                'two help',
+                )
         registry.set_default('one')
         opt = option.RegistryOption('format', 'format help', registry,
                                     value_switches=False)

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2007-02-13 20:33:57 +0000
+++ bzrlib/tests/test_osutils.py	2007-02-17 01:42:57 +0000
@@ -26,6 +26,7 @@
 from bzrlib import (
     errors,
     osutils,
+    win32utils,
     )
 from bzrlib.errors import BzrBadParameterNotUnicode, InvalidURL
 from bzrlib.tests import (
@@ -314,6 +315,8 @@
     def test_abspath(self):
         self.assertEqual('C:/foo', osutils._win32_abspath('C:\\foo'))
         self.assertEqual('C:/foo', osutils._win32_abspath('C:/foo'))
+        self.assertEqual('//HOST/path', osutils._win32_abspath(r'\\HOST\path'))
+        self.assertEqual('//HOST/path', osutils._win32_abspath('//HOST/path'))
 
     def test_realpath(self):
         self.assertEqual('C:/foo', osutils._win32_realpath('C:\\foo'))
@@ -347,11 +350,29 @@
         self.assertEqual('H:/foo', osutils._win32_fixdrive('H:/foo'))
         self.assertEqual('C:\\foo', osutils._win32_fixdrive('c:\\foo'))
 
+    def test_win98_abspath(self):
+        # absolute path
+        self.assertEqual('C:/foo', osutils._win98_abspath('C:\\foo'))
+        self.assertEqual('C:/foo', osutils._win98_abspath('C:/foo'))
+        # UNC path
+        self.assertEqual('//HOST/path', osutils._win98_abspath(r'\\HOST\path'))
+        self.assertEqual('//HOST/path', osutils._win98_abspath('//HOST/path'))
+        # relative path
+        cwd = osutils.getcwd().rstrip('/')
+        drive = osutils._nt_splitdrive(cwd)[0]
+        self.assertEqual(cwd+'/path', osutils._win98_abspath('path'))
+        self.assertEqual(drive+'/path', osutils._win98_abspath('/path'))
+        # unicode path
+        u = u'\u1234'
+        self.assertEqual(cwd+'/'+u, osutils._win98_abspath(u))
+
 
 class TestWin32FuncsDirs(TestCaseInTempDir):
     """Test win32 functions that create files."""
     
     def test_getcwd(self):
+        if win32utils.winver == 'Windows 98':
+            raise TestSkipped('Windows 98 cannot handle unicode filenames')
         # Make sure getcwd can handle unicode filenames
         try:
             os.mkdir(u'mu-\xb5')

=== modified file 'bzrlib/tests/test_testament.py'
--- bzrlib/tests/test_testament.py	2006-10-16 01:50:48 +0000
+++ bzrlib/tests/test_testament.py	2007-02-09 20:35:15 +0000
@@ -31,7 +31,7 @@
 
     def setUp(self):
         super(TestamentSetup, self).setUp()
-        self.wt = self.make_branch_and_tree('.')
+        self.wt = self.make_branch_and_tree('.', format='experimental-knit2')
         self.wt.set_root_id('TREE_ROT')
         b = self.b = self.wt.branch
         b.nick = "test branch"
@@ -281,7 +281,7 @@
 message:
   add files and directories
 inventory:
-  directory . TREE_ROT test@user-2 no
+  directory . TREE_ROT test@user-1 no
   file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
   directory src src-id test@user-2 no
   file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no
@@ -375,7 +375,7 @@
 message:
   revision with properties
 inventory:
-  directory . TREE_ROT test@user-3 no
+  directory . TREE_ROT test@user-1 no
   file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
   directory src src-id test@user-2 no
   file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no
@@ -444,7 +444,7 @@
 message:
   add symlink
 inventory:
-  directory . TREE_ROT test@user-3 no
+  directory . TREE_ROT test@user-1 no
   file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
   symlink link link-id wibble/linktarget test@user-3 no
   directory src src-id test@user-2 no
@@ -510,7 +510,7 @@
 message:
   non-ascii commit \N{COPYRIGHT SIGN} me
 inventory:
-  directory . TREE_ROT test@user-3 no
+  directory . TREE_ROT test@user-1 no
   file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
   directory src src-id test@user-2 no
   file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2006-11-02 22:48:49 +0000
+++ bzrlib/tests/test_transform.py	2007-02-09 15:28:12 +0000
@@ -587,6 +587,155 @@
         bar1_abspath = self.wt.abspath('bar')
         self.assertEqual([bar1_abspath], stat_paths)
 
+    def test_iter_changes(self):
+        transform, root = self.get_transform()
+        transform.new_file('old', root, 'blah', 'id-1', True)
+        transform.apply()
+        transform, root = self.get_transform()
+        try:
+            self.assertEqual([], list(transform._iter_changes()))
+            old = transform.trans_id_tree_file_id('id-1')
+            transform.unversion_file(old)
+            self.assertEqual([('id-1', 'old', False, (True, False),
+                ('TREE_ROOT', 'TREE_ROOT'), ('old', 'old'), ('file', 'file'),
+                (True, True))], list(transform._iter_changes()))
+            transform.new_directory('new', root, 'id-1')
+            self.assertEqual([('id-1', 'new', True, (True, True),
+                ('TREE_ROOT', 'TREE_ROOT'), ('old', 'new'),
+                ('file', 'directory'),
+                (True, False))], list(transform._iter_changes()))
+        finally:
+            transform.finalize()
+
+    def test_iter_changes_new(self):
+        transform, root = self.get_transform()
+        transform.new_file('old', root, 'blah')
+        transform.apply()
+        transform, root = self.get_transform()
+        try:
+            old = transform.trans_id_tree_path('old')
+            transform.version_file('id-1', old)
+            self.assertEqual([('id-1', 'old', False, (False, True),
+                ('TREE_ROOT', 'TREE_ROOT'), ('old', 'old'), ('file', 'file'),
+                (False, False))], list(transform._iter_changes()))
+        finally:
+            transform.finalize()
+
+    def test_iter_changes_modifications(self):
+        transform, root = self.get_transform()
+        transform.new_file('old', root, 'blah', 'id-1')
+        transform.new_file('new', root, 'blah')
+        transform.new_directory('subdir', root, 'subdir-id')
+        transform.apply()
+        transform, root = self.get_transform()
+        try:
+            old = transform.trans_id_tree_path('old')
+            subdir = transform.trans_id_tree_file_id('subdir-id')
+            new = transform.trans_id_tree_path('new')
+            self.assertEqual([], list(transform._iter_changes()))
+
+            #content deletion
+            transform.delete_contents(old)
+            self.assertEqual([('id-1', 'old', True, (True, True),
+                ('TREE_ROOT', 'TREE_ROOT'), ('old', 'old'), ('file', None),
+                (False, False))], list(transform._iter_changes()))
+
+            #content change
+            transform.create_file('blah', old)
+            self.assertEqual([('id-1', 'old', True, (True, True),
+                ('TREE_ROOT', 'TREE_ROOT'), ('old', 'old'), ('file', 'file'),
+                (False, False))], list(transform._iter_changes()))
+            transform.cancel_deletion(old)
+            self.assertEqual([('id-1', 'old', True, (True, True),
+                ('TREE_ROOT', 'TREE_ROOT'), ('old', 'old'), ('file', 'file'),
+                (False, False))], list(transform._iter_changes()))
+            transform.cancel_creation(old)
+
+            # move file_id to a different file
+            self.assertEqual([], list(transform._iter_changes()))
+            transform.unversion_file(old)
+            transform.version_file('id-1', new)
+            transform.adjust_path('old', root, new)
+            self.assertEqual([('id-1', 'old', True, (True, True),
+                ('TREE_ROOT', 'TREE_ROOT'), ('old', 'old'), ('file', 'file'),
+                (False, False))], list(transform._iter_changes()))
+            transform.cancel_versioning(new)
+            transform._removed_id = set()
+
+            #execute bit
+            self.assertEqual([], list(transform._iter_changes()))
+            transform.set_executability(True, old)
+            self.assertEqual([('id-1', 'old', False, (True, True),
+                ('TREE_ROOT', 'TREE_ROOT'), ('old', 'old'), ('file', 'file'),
+                (False, True))], list(transform._iter_changes()))
+            transform.set_executability(None, old)
+
+            # filename
+            self.assertEqual([], list(transform._iter_changes()))
+            transform.adjust_path('new', root, old)
+            transform._new_parent = {}
+            self.assertEqual([('id-1', 'new', False, (True, True),
+                ('TREE_ROOT', 'TREE_ROOT'), ('old', 'new'), ('file', 'file'),
+                (False, False))], list(transform._iter_changes()))
+            transform._new_name = {}
+
+            # parent directory
+            self.assertEqual([], list(transform._iter_changes()))
+            transform.adjust_path('new', subdir, old)
+            transform._new_name = {}
+            self.assertEqual([('id-1', 'subdir/old', False, (True, True),
+                ('TREE_ROOT', 'subdir-id'), ('old', 'old'), ('file', 'file'),
+                (False, False))], list(transform._iter_changes()))
+            transform._new_path = {}
+
+        finally:
+            transform.finalize()
+
+    def test_iter_changes_modified_bleed(self):
+        """Modified flag should not bleed from one change to another"""
+        # unfortunately, we have no guarantee that file1 (which is modified)
+        # will be applied before file2.  And if it's applied after file2, it
+        # obviously can't bleed into file2's change output.  But for now, it
+        # works.
+        transform, root = self.get_transform()
+        transform.new_file('file1', root, 'blah', 'id-1')
+        transform.new_file('file2', root, 'blah', 'id-2')
+        transform.apply()
+        transform, root = self.get_transform()
+        try:
+            transform.delete_contents(transform.trans_id_file_id('id-1'))
+            transform.set_executability(True,
+            transform.trans_id_file_id('id-2'))
+            self.assertEqual([('id-1', u'file1', True, (True, True),
+                ('TREE_ROOT', 'TREE_ROOT'), ('file1', u'file1'),
+                ('file', None), (False, False)),
+                ('id-2', u'file2', False, (True, True),
+                ('TREE_ROOT', 'TREE_ROOT'), ('file2', u'file2'),
+                ('file', 'file'), (False, True))],
+                list(transform._iter_changes()))
+        finally:
+            transform.finalize()
+
+    def test_iter_changes_pointless(self):
+        """Ensure that no-ops are not treated as modifications"""
+        transform, root = self.get_transform()
+        transform.new_file('old', root, 'blah', 'id-1')
+        transform.new_directory('subdir', root, 'subdir-id')
+        transform.apply()
+        transform, root = self.get_transform()
+        try:
+            old = transform.trans_id_tree_path('old')
+            subdir = transform.trans_id_tree_file_id('subdir-id')
+            self.assertEqual([], list(transform._iter_changes()))
+            transform.delete_contents(subdir)
+            transform.create_directory(subdir)
+            transform.set_executability(False, old)
+            transform.unversion_file(old)
+            transform.version_file('id-1', old)
+            transform.adjust_path('old', root, old)
+            self.assertEqual([], list(transform._iter_changes()))
+        finally:
+            transform.finalize()
 
 class TransformGroup(object):
     def __init__(self, dirname, root_id):
@@ -598,6 +747,7 @@
         self.tt = TreeTransform(self.wt)
         self.root = self.tt.trans_id_tree_file_id(self.wt.get_root_id())
 
+
 def conflict_text(tree, merge):
     template = '%s TREE\n%s%s\n%s%s MERGE-SOURCE\n'
     return template % ('<' * 7, tree, '=' * 7, merge, '>' * 7)

=== modified file 'bzrlib/tests/test_upgrade.py'
--- bzrlib/tests/test_upgrade.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/test_upgrade.py	2007-02-09 22:59:51 +0000
@@ -26,10 +26,14 @@
 import os
 import sys
 
+from bzrlib import (
+    branch as _mod_branch,
+    bzrdir,
+    progress,
+    repository,
+    )
 import bzrlib.branch
 from bzrlib.branch import Branch
-import bzrlib.bzrdir as bzrdir
-import bzrlib.repository as repository
 from bzrlib.revision import is_ancestor
 from bzrlib.tests import TestCase, TestCaseInTempDir
 from bzrlib.transport import get_transport
@@ -159,6 +163,28 @@
            ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
             'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd'])
 
+    def test_convert_branch5_branch6(self):
+        branch = self.make_branch('branch', format='knit')
+        branch.set_revision_history(['AB', 'CD'])
+        branch.set_parent('file:///EF')
+        branch.set_bound_location('file:///GH')
+        branch.set_push_location('file:///IJ')
+        target = bzrdir.format_registry.make_bzrdir('experimental-branch6')
+        converter = branch.bzrdir._format.get_converter(target)
+        converter.convert(branch.bzrdir, progress.DummyProgress())
+        new_branch = _mod_branch.Branch.open(self.get_url('branch'))
+        self.assertIs(new_branch.__class__, _mod_branch.BzrBranch6)
+        self.assertEqual('CD', new_branch.last_revision())
+        self.assertEqual('file:///EF', new_branch.get_parent())
+        self.assertEqual('file:///GH', new_branch.get_bound_location())
+        branch_config = new_branch.get_config()._get_branch_data_config()
+        self.assertEqual('file:///IJ',
+            branch_config.get_user_option('push_location'))
+
+        branch2 = self.make_branch('branch2', format='knit')
+        converter = branch2.bzrdir._format.get_converter(target)
+        converter.convert(branch2.bzrdir, progress.DummyProgress())
+
 
 _upgrade1_template = \
      [

=== modified file 'bzrlib/tests/test_urlutils.py'
--- bzrlib/tests/test_urlutils.py	2006-12-07 16:16:23 +0000
+++ bzrlib/tests/test_urlutils.py	2007-02-14 10:11:12 +0000
@@ -20,7 +20,7 @@
 import re
 import sys
 
-from bzrlib import osutils, urlutils
+from bzrlib import osutils, urlutils, win32utils
 import bzrlib
 from bzrlib.errors import InvalidURL, InvalidURLJoin
 from bzrlib.tests import TestCaseInTempDir, TestCase, TestSkipped
@@ -506,6 +506,9 @@
         self.assertEndsWith(url, '/mytest')
 
     def test_non_ascii(self):
+        if win32utils.winver == 'Windows 98':
+            raise TestSkipped('Windows 98 cannot handle unicode filenames')
+
         try:
             os.mkdir(u'dod\xe9')
         except UnicodeError:

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2007-02-09 17:24:23 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2007-02-17 01:42:57 +0000
@@ -288,7 +288,12 @@
         # and now we can set it to 'A'
         # because some formats mutate the branch to set it on the tree
         # we need to alter the branch to let this pass.
-        wt.branch.set_revision_history(['A', 'B'])
+        try:
+            wt.branch.set_revision_history(['A', 'B'])
+        except errors.NoSuchRevision, e:
+            self.assertEqual('B', e.revision)
+            raise TestSkipped("Branch format does not permit arbitrary"
+                              " history")
         wt.set_last_revision('A')
         self.assertEqual(['A'], wt.get_parent_ids())
         self.assertRaises(errors.ReservedId, wt.set_last_revision, 'A:')

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2007-02-01 23:48:08 +0000
+++ bzrlib/transform.py	2007-02-14 21:40:53 +0000
@@ -19,6 +19,10 @@
 from stat import S_ISREG
 
 from bzrlib import bzrdir, errors
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import delta
+""")
 from bzrlib.errors import (DuplicateKey, MalformedTransform, NoSuchFile,
                            ReusingTransform, NotVersionedError, CantMoveRoot,
                            ExistingLimbo, ImmortalLimbo, NoFinalPath)
@@ -903,6 +907,139 @@
         self.create_symlink(target, trans_id)
         return trans_id
 
+    def _affected_ids(self):
+        """Return the set of transform ids affected by the transform"""
+        trans_ids = set(self._removed_id)
+        trans_ids.update(self._new_id.keys())
+        trans_ids.update(self._removed_contents)
+        trans_ids.update(self._new_contents.keys())
+        trans_ids.update(self._new_executability.keys())
+        trans_ids.update(self._new_name.keys())
+        trans_ids.update(self._new_parent.keys())
+        return trans_ids
+
+    def _get_file_id_maps(self):
+        """Return mapping of file_ids to trans_ids in the to and from states"""
+        trans_ids = self._affected_ids()
+        from_trans_ids = {}
+        to_trans_ids = {}
+        # Build up two dicts: trans_ids associated with file ids in the
+        # FROM state, vs the TO state.
+        for trans_id in trans_ids:
+            from_file_id = self.tree_file_id(trans_id)
+            if from_file_id is not None:
+                from_trans_ids[from_file_id] = trans_id
+            to_file_id = self.final_file_id(trans_id)
+            if to_file_id is not None:
+                to_trans_ids[to_file_id] = trans_id
+        return from_trans_ids, to_trans_ids
+
+    def _from_file_data(self, from_trans_id, from_versioned, file_id):
+        """Get data about a file in the from (tree) state
+
+        Return a (name, parent, kind, executable) tuple
+        """
+        from_path = self._tree_id_paths.get(from_trans_id)
+        if from_versioned:
+            # get data from working tree if versioned
+            from_entry = self._tree.inventory[file_id]
+            from_name = from_entry.name
+            from_parent = from_entry.parent_id
+        else:
+            from_entry = None
+            if from_path is None:
+                # File does not exist in FROM state
+                from_name = None
+                from_parent = None
+            else:
+                # File exists, but is not versioned.  Have to use path-
+                # splitting stuff
+                from_name = os.path.basename(from_path)
+                tree_parent = self.get_tree_parent(from_trans_id)
+                from_parent = self.tree_file_id(tree_parent)
+        if from_path is not None:
+            from_kind, from_executable, from_stats = \
+                self._tree._comparison_data(from_entry, from_path)
+        else:
+            from_kind = None
+            from_executable = False
+        return from_name, from_parent, from_kind, from_executable
+
+    def _to_file_data(self, to_trans_id, from_trans_id, from_executable):
+        """Get data about a file in the to (target) state
+
+        Return a (name, parent, kind, executable) tuple
+        """
+        to_name = self.final_name(to_trans_id)
+        try:
+            to_kind = self.final_kind(to_trans_id)
+        except NoSuchFile:
+            to_kind = None
+        to_parent = self.final_file_id(self.final_parent(to_trans_id))
+        if to_trans_id in self._new_executability:
+            to_executable = self._new_executability[to_trans_id]
+        elif to_trans_id == from_trans_id:
+            to_executable = from_executable
+        else:
+            to_executable = False
+        return to_name, to_parent, to_kind, to_executable
+
+    def _iter_changes(self):
+        """Produce output in the same format as Tree._iter_changes.
+
+        Will produce nonsensical results if invoked while inventory/filesystem
+        conflicts (as reported by TreeTransform.find_conflicts()) are present.
+
+        This reads the Transform, but only reproduces changes involving a
+        file_id.  Files that are not versioned in either of the FROM or TO
+        states are not reflected.
+        """
+        final_paths = FinalPaths(self)
+        from_trans_ids, to_trans_ids = self._get_file_id_maps()
+        results = []
+        # Now iterate through all active file_ids
+        for file_id in set(from_trans_ids.keys() + to_trans_ids.keys()):
+            modified = False
+            from_trans_id = from_trans_ids.get(file_id)
+            # find file ids, and determine versioning state
+            if from_trans_id is None:
+                from_versioned = False
+                from_trans_id = to_trans_ids[file_id]
+            else:
+                from_versioned = True
+            to_trans_id = to_trans_ids.get(file_id)
+            if to_trans_id is None:
+                to_versioned = False
+                to_trans_id = from_trans_id
+            else:
+                to_versioned = True
+
+            from_name, from_parent, from_kind, from_executable = \
+                self._from_file_data(from_trans_id, from_versioned, file_id)
+
+            to_name, to_parent, to_kind, to_executable = \
+                self._to_file_data(to_trans_id, from_trans_id, from_executable)
+
+            to_path = final_paths.get_path(to_trans_id)
+            if from_kind != to_kind:
+                modified = True
+            elif to_kind in ('file' or 'symlink') and (
+                to_trans_id != from_trans_id or
+                to_trans_id in self._new_contents):
+                modified = True
+            if (not modified and from_versioned == to_versioned and
+                from_parent==to_parent and from_name == to_name and
+                from_executable == to_executable):
+                continue
+            results.append((file_id, to_path, modified,
+                   (from_versioned, to_versioned),
+                   (from_parent, to_parent),
+                   (from_name, to_name),
+                   (from_kind, to_kind),
+                   (from_executable, to_executable)))
+        return iter(sorted(results, key=lambda x:x[1]))
+
+
 def joinpath(parent, child):
     """Join tree-relative paths, handling the tree root specially"""
     if parent is None or parent == "":
@@ -1208,7 +1345,7 @@
         child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
         try:
             _alter_files(working_tree, target_tree, tt, child_pb, 
-                         interesting_ids, backups, change_reporter)
+                         interesting_ids, backups)
         finally:
             child_pb.finished()
         pp.next_phase()
@@ -1218,6 +1355,9 @@
         finally:
             child_pb.finished()
         conflicts = cook_conflicts(raw_conflicts, tt)
+        if change_reporter:
+            change_reporter = delta.ChangeReporter(working_tree.inventory)
+            delta.report_changes(tt._iter_changes(), change_reporter)
         for conflict in conflicts:
             warning(conflict)
         pp.next_phase()
@@ -1229,22 +1369,18 @@
     return conflicts
 
 
-def _alter_files(working_tree, target_tree, tt, pb, interesting_ids, backups,
-                 report_changes):
-    from bzrlib import delta
+def _alter_files(working_tree, target_tree, tt, pb, interesting_ids,
+                 backups):
     merge_modified = working_tree.merge_modified()
-    change_list = list(target_tree._iter_changes(working_tree,
-        specific_file_ids=interesting_ids, pb=pb))
+    change_list = target_tree._iter_changes(working_tree,
+        specific_file_ids=interesting_ids, pb=pb)
     if target_tree.inventory.root is None:
         skip_root = True
     else:
         skip_root = False
     basis_tree = None
-    if report_changes:
-        change_reporter = delta.ChangeReporter(working_tree.inventory)
-        delta.report_changes(change_list, change_reporter)
-    for id_num, (file_id, path, changed_content, versioned, parent, name, kind,
-                 executable) in enumerate(change_list):
+    for id_num, (file_id, path, changed_content, versioned, parent, name,
+        kind, executable) in enumerate(change_list):
         if skip_root and file_id[0] is not None and parent[0] is None:
             continue
         trans_id = tt.trans_id_file_id(file_id)

=== modified file 'bzrlib/transport/smart.py'
--- bzrlib/transport/smart.py	2006-12-18 15:34:08 +0000
+++ bzrlib/transport/smart.py	2007-02-15 06:15:43 +0000
@@ -867,9 +867,11 @@
 
     def stop_background_thread(self):
         self._should_terminate = True
+        # At one point we would wait to join the threads here, but it looks
+        # like they don't actually exit.  So now we just leave them running
+        # and expect to terminate the process. -- mbp 20070215
         # self._server_socket.close()
-        # we used to join the thread, but it's not really necessary; it will
-        # terminate in time
+        ## sys.stderr.write("waiting for server thread to finish...")
         ## self._server_thread.join()
 
 

=== modified file 'bzrlib/urlutils.py'
--- bzrlib/urlutils.py	2007-01-28 21:43:32 +0000
+++ bzrlib/urlutils.py	2007-02-14 10:11:12 +0000
@@ -212,8 +212,7 @@
     #       which actually strips trailing space characters.
     #       The worst part is that under linux ntpath.abspath has different
     #       semantics, since 'nt' is not an available module.
-    win32_path = osutils._nt_normpath(
-        osutils._win32_abspath(path)).replace('\\', '/')
+    win32_path = osutils._win32_abspath(path)
     # check for UNC path \\HOST\path
     if win32_path.startswith('//'):
         return 'file:' + escape(win32_path)

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2007-02-13 20:33:57 +0000
+++ bzrlib/workingtree.py	2007-02-17 01:42:57 +0000
@@ -1319,7 +1319,8 @@
                 yield stem
 
     @needs_write_lock
-    def pull(self, source, overwrite=False, stop_revision=None):
+    def pull(self, source, overwrite=False, stop_revision=None,
+             change_reporter=None):
         top_pb = bzrlib.ui.ui_factory.nested_progress_bar()
         source.lock_read()
         try:
@@ -1340,7 +1341,8 @@
                                 new_basis_tree,
                                 basis_tree,
                                 this_tree=self,
-                                pb=pb)
+                                pb=pb,
+                                change_reporter=change_reporter)
                     if (basis_tree.inventory.root is None and
                         new_basis_tree.inventory.root is not None):
                         self.set_root_id(new_basis_tree.inventory.root.file_id)

=== modified file 'doc/configuration.txt'
--- doc/configuration.txt	2007-01-02 07:16:27 +0000
+++ doc/configuration.txt	2007-02-13 22:34:57 +0000
@@ -181,3 +181,39 @@
 
     gpg_signing_command = /usr/bin/gnpg
 
+
+Branch 6 Options
+================
+
+These options apply only to branches that use the "experimental-branch6"
+format.  They are usually set in .bzr/branch/branch.conf automatically, but
+may be manually set in locations.conf or bazaar.conf
+
+append_revisions_only
+---------------------
+If set to "True", then revisions can only be appended to the log, not removed.
+A branch with this setting enabled can only pull from another branch if the
+other branch's log is a longer version of its own.
+
+This is normally set by ``bzr init --append-revisions-only``.
+
+parent_location
+---------------
+If present, the location of the default branch for pull or merge.
+
+This is normally set by ``pull --remember`` or ``merge --remember``
+
+push_location
+-------------
+If present, the location of the default branch for push.  This is normally
+set by ``push --remember``.
+
+bound_location
+--------------
+The location that commits should go to when acting as a checkout.  (Normally
+set by ``bind``)
+
+bound
+-----
+If set to "True", the branch should act as a checkout, and push each commit to
+the bound_location.  (Normally set by ``bind``/``unbind``.)

